package de.bernhardjung.dsaprogramm.basis;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public class GeldBoerse {

    private int dukaten;
    private int silbertaler;
    private int heller;
    private int kreuzer;
    
    public static boolean initiiert = false;
    
    //Initialisiert geldboerse nach erstellung des Helden
    public GeldBoerse( int so, VorNachteilWert wert ) {
        
        int gesStartKapital = 0;
        
        if( wert == null ) {
            
            gesStartKapital = ( ( int ) Math.pow( so, 2 ) ) * 100;
        } else {
         
            gesStartKapital = ( ( (int) Math.pow( so, 2 ) * 100 ) + 
                    ( 2 * so * wert.getStufe() * 1000 ) );
        }
        
        dukaten = gesStartKapital / 1000;
        silbertaler = ( gesStartKapital % 1000 ) / 100;
        heller = ( gesStartKapital % 100 ) / 10;
        kreuzer = ( gesStartKapital % 10 );
        
        initiiert = true;
    }

    // Laedt Geldbetrag aus XML (sofern vorhanden)
    public GeldBoerse( int k ) {

        dukaten = k / 1000;
        silbertaler = ( k % 1000 ) / 100;
        heller = ( k % 100 ) / 10;
        kreuzer = ( k % 10 );
        
        initiiert = true;
    }

    // initialisiert Geldboerse mit einem String
    public GeldBoerse( String str ) throws Exception {
        this( parseMoneyString( str ) );
        initiiert = true;
    }

    private static int parseMoneyString( String str ) throws Exception {

        int gesVermoegen = 0; // haellt das gesVerm aus str
        int tmpVal;

        str = str.toLowerCase(); // String normalisieren

        Pattern regex = Pattern.compile( "(\\d+)\\s*(\\w)\\s*" ); // ges.: eine
                                                                  // zahl
                                                                  // gefolgt
                                                                  // von >= 0
                                                                  // leerzeichen,
                                                                  // gefolgt von
                                                                  // einem
                                                                  // Buchstaben,
                                                                  // gefolgt von
                                                                  // >= 0
                                                                  // leerzeichen

        Matcher m = regex.matcher( str );

        while ( !m.hitEnd() ) {

            if ( m.find() ) {

                try {

                    tmpVal = Integer.parseInt( m.group( 1 ) );

                    if ( m.group( 2 ).equals( "d" ) ) {

                        gesVermoegen += ( tmpVal * 1000 );
                    }

                    else if ( m.group( 2 ).equals( "s" ) ) {

                        gesVermoegen += tmpVal * 100;

                    }

                    else if ( m.group( 2 ).equals( "h" ) ) {

                        gesVermoegen += tmpVal * 10;

                    }

                    else if ( m.group( 2 ).equals( "k" ) ) {

                        gesVermoegen += tmpVal;

                    }

                    else {

                        throw new Exception(
                                "falsche oder keine Wert-identifizierung angegeben!" );

                    }

                } catch ( NumberFormatException nfe ) {

                    throw new Exception(
                            "Fehler: Konnte keinen int parsen. Falschen Wert eingegeben?" );
                }

            }

        }

        return gesVermoegen;
    }

    public int getDukaten() {

        return dukaten;

    }

    public void setDukaten( int d ) {

        dukaten = d;

    }

    public int getSilbertaler() {

        return silbertaler;

    }

    public void setSilbertaler( int s ) {

        silbertaler = s;

    }

    public int getHeller() {

        return heller;

    }

    public void setHeller( int h ) {

        heller = h;

    }

    public int getKreuzer() {

        return kreuzer;

    }

    public void setKreuzer( int k ) {

        kreuzer = k;

    }

    /**
     * Overload, noetig fuer einfacheres hantieren beim speichern
     * @return gesVermoegen in kreuzern
     */
    
    public int umrechnen() {
        
        int gesVermoegen = 0;
        int k = getKreuzer();
        int h = getHeller();
        int s = getSilbertaler();
        int d = getDukaten();
        
        gesVermoegen += k;
        gesVermoegen += ( h * 10 );
        gesVermoegen += ( s * 100 );
        gesVermoegen += ( d * 1000 );
        
        return gesVermoegen;
    }
    /**
     * Diese Methode (wie auch die naechsten) sind "vorsorglich" erstellt. Sie
     * koennen durchaus nuetzlich sein, wenn es mehrere waehrungen gibt.
     * 
     * @param d
     *            Dukaten des Charakters
     * @param s
     *            Silbertaler des Charakters
     * @param h
     *            Heller des Charakters
     * @param k
     *            Kreuzer des Charakters
     * @return <code>gesVermoegen</code> in kreuzern
     */

    public int umrechnen( int d, int s, int h, int k ) {

        int gesVermoegen = 0;
        // 1 D == 10 S == 100 H == 1000 K

        gesVermoegen += k;
        gesVermoegen += ( h * 10 );
        gesVermoegen += ( s * 100 );
        gesVermoegen += ( d * 1000 );

        return gesVermoegen;
    }

    /**
     * @param gesVermoegen
     *            gesamtes Vermoegen in kreuzern
     * @return kreuzer-anteil von gesVermoegen
     */

    public int kreuzerAusVermoegen( int gesVermoegen ) {

        return ( gesVermoegen % 10 );
    }

    /**
     * @param gesVermoegen
     *            gesamtes Vermoegen in Kreuzern
     * @return heller-Anteil von gesVermoegen
     */

    public int hellerAusVermoegen( int gesVermoegen ) {

        return ( gesVermoegen % 100 ) / 10;
    }

    /**
     * @param gesVermoegen
     *            gesamtes Vermoegen in Kreuzern
     * @return Silberstuecke-Anteil von gesVermoegen
     */

    public int silberAusVermoegen( int gesVermoegen ) {

        return ( gesVermoegen % 1000 ) / 100;
    }

    /**
     * @param gesVermoegen
     *            gesamtes Vermoegen in kreuzern
     * @return Dukaten-Anteil von gesVermoegen
     */

    public int dukatenAusVermoegen( int gesVermoegen ) {

        return gesVermoegen / 1000;
    }

    public void betragHinzufuegen( String str ) throws Exception {

        int betrag = parseMoneyString( str );
        int gesVermoegen = umrechnen( getDukaten(), getSilbertaler(),
                getHeller(), getKreuzer() );
        gesVermoegen += betrag;

        setDukaten( dukatenAusVermoegen( gesVermoegen ) );
        setSilbertaler( silberAusVermoegen( gesVermoegen ) );
        setHeller( hellerAusVermoegen( gesVermoegen ) );
        setKreuzer( kreuzerAusVermoegen( gesVermoegen ) );

    }

    public void betragEntfernen( String str ) throws Exception {

        int betrag = parseMoneyString( str );

        int gesVermoegen = umrechnen( getDukaten(), getSilbertaler(),
                getHeller(), getKreuzer() );

        if ( transaktionValidieren( betrag, gesVermoegen ) ) {
            gesVermoegen -= betrag;
            setDukaten( dukatenAusVermoegen( gesVermoegen ) );
            setSilbertaler( silberAusVermoegen( gesVermoegen ) );
            setHeller( hellerAusVermoegen( gesVermoegen ) );
            setKreuzer( kreuzerAusVermoegen( gesVermoegen ) );
        } else {

            throw new Exception("Nicht genug Vermoegen, um diese Transaktion auszufuehren!");

        }

    }

    public boolean transaktionValidieren( int betrag, int gesVermoegen ) {

        return ( gesVermoegen >= betrag ? true : false );
    }

    @Override
    public String toString() {

        return getDukaten() + " Dukaten, " + getSilbertaler() + " Silertaler, "
                + getHeller() + " Heller, " + getKreuzer() + " kreuzer.";

    }

}

