/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;

public class VariantenBesondererBesitz implements Varianten {

    /**
     * Macht nichts.
     */
    public void einfuegen( final Variante variante ) {
        // ignorieren.
    }
    
    /**
     * Wenn nur ein String uebergeben wird, wird versucht einen 
     * Lokalisierungsschluessel dafuer zu ermitteln.
     */
    public Variante getVariante( final Object objekt ) {
        
        if ( objekt instanceof Variante ) {
            
            return (Variante) objekt;
        }
        
        final List< Profession > professionen =
            Datenspeicher.getInstanz().getProfessionen();
        
        for ( final Profession profession : professionen ) {
            
            for ( final Variante besondererBesitz : profession.getBesondererBesitz() ) {
                
                if ( besondererBesitz.getName().equals( objekt ) ) {
                    
                    return besondererBesitz;
                }
            }   
        }
        
        return new Variante( (String) objekt );
    }
    
    /**
     * @return die Moeglichkeiten, die von der Profession des Helden,
     *         vorgegeben werden.
     */
    @SuppressWarnings( "unchecked" )
    public List< Variante > getVarianten( final Held held ) {

        if ( null != held ) {

            return held.getProfession().getBesondererBesitz();
            
        } else {

            return Collections.EMPTY_LIST;
        }
    }

    /**
     * @return {@code true}
     */
    public boolean hatVarianten() {
        return true;
    }

    /**
     * @return {@code true}
     */
    public boolean istFreieVariante() {
        return true;
    }
}
