/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.Held;

public class VariantenFrei implements Varianten {

    /** Die Varianten die zur Verfuegung stehen. */
    public final List< Variante > varianten;

    /** Erzeugt mit den uebergebenen Varianten eine neue Liste. */
    public VariantenFrei( final String ... dieVarianten ) {

        varianten = new ArrayList< Variante >( dieVarianten.length );

        for ( final String variante : dieVarianten ) {

            varianten.add( new Variante( variante ) );
        }

        Collections.sort( varianten, Variante.NAME_COMPARATOR );
    }

    /** Erzeugt mit den uebergebenen Varianten eine neue Liste. */
    public VariantenFrei( final Collection< String > dieVarianten ) {

        varianten = new ArrayList< Variante >();
        
        for ( final String variante : dieVarianten ) {
            
            varianten.add( new Variante( variante ) );
        }

        Collections.sort( varianten, Variante.NAME_COMPARATOR );
    }

    /** {@inheritDoc} */
    public void einfuegen( Variante variante ) {

        if ( variante.isNull() ) {
            
            return;
        }
        
        int index = Collections.binarySearch( 
                varianten, 
                variante,
                Variante.NAME_COMPARATOR );

        if ( index < 0 ) {
            // Die Varianten ist noch nicht in der Liste.
            index = -( index + 1 );

            varianten.add( index, variante );
        }
    }

    /**
     * Wird ein String uebergeben, wird er als Variante verpackt.
     */
    public Variante getVariante( final Object objekt ) {
        
        if ( objekt instanceof ElementMitName ) {
            
            return new Variante( (ElementMitName) objekt );
        }
        
        final Variante variante;
        
        if ( objekt instanceof Variante ) {
            
            variante = (Variante) objekt;
        
        } else {
            
            variante = new Variante( (String) objekt );
        }
        
        if ( VariantenUtil.enthaeltVarianteMitNamen( varianten, variante ) ) {
            
            return VariantenUtil.getVarianteMitNamen( varianten, variante );
        
        } else {
            
            einfuegen( variante );
            
            return variante;
        }
    }
    
    /**
     * @param held wird nicht benoetigt und darf {@code null} sein.
     * {@inheritDoc} */
    public List< Variante > getVarianten(final Held held) {
        return new ArrayList< Variante >( varianten );
    }

    /** @return {@code true} */
    public boolean hatVarianten() {
        return true;
    }

    /** @return {@code true} */
    public boolean istFreieVariante() {
        return true;
    }

    /** Eine Aufzaehlung aller Varianten. */
    @Override
    public final String toString() {
        return "freie Varianten: " + getVarianten( null ).toString();
    }
}
