/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;

public class VariantenTalentspezialisierung implements Varianten {

    private final List< Variante > varianten;
    
    public VariantenTalentspezialisierung( 
            final List< Variante > spezialisierungen ) {
        
        varianten = new ArrayList< Variante >( spezialisierungen );
    }
    
    public void einfuegen( Variante neueVariante ) {
        
        if ( neueVariante.isNull() ) {
            
            return;
        }
        
        if ( ! VariantenUtil.enthaeltVarianteMitNamen( 
                varianten, 
                neueVariante ) ) {
            
            varianten.add( neueVariante );
        }
    }
    
    public Variante getVariante( final Object objekt ) {
        
        final Variante variante;
        
        if ( objekt instanceof Variante ) {
            
            variante = (Variante) objekt;
        
        } else {
            
            variante = new Variante( (String) objekt ); 
        }
        
        if ( VariantenUtil.enthaeltVarianteMitNamen( varianten, variante ) ) {
            
            return VariantenUtil.getVarianteMitNamen( varianten, variante );
        
        } else {
            
            einfuegen( variante );
            
            return variante;
        }
    }
    
    public List< Variante > getVarianten( Held held ) {

        Collections.sort( varianten, Variante.NAME_COMPARATOR );
        
        return new ArrayList< Variante >( varianten );
    }

    public boolean hatVarianten() {
        
        return true;
    }

    public boolean istFreieVariante() {
        
        return true;
    }

}
