/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basiswerte;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;

class Formel {

    /**
     * Die Zahl durch die die Summe der Eigenschaftswerte geteilt wird.
     */
    private final float divisor;

    /**
     * Die Eigenschaften die fuer die Formel aufaddiert werden.
     */
    private final List< Eigenschaft > eigenschaften;

    /**
     * @param derDivisor durch den die Summe der Eigenschaften geteilt wird.
     * @param dieEigenschaften die zum Berechnen der Formel aufaddiert werden.
     */
    Formel( final float derDivisor, final Eigenschaft ... dieEigenschaften ) {

        divisor = derDivisor;

        eigenschaften = Arrays.asList( dieEigenschaften );
    }

    /**
     * Berechnet die Formel fuer die Eigenschaften eines Helden.
     *
     * @param held fuer den die Formel berechnet werden soll.
     * @return das Ergebnis der Berechnung der Formel.
     */
    final int berechnen( final Held held ) {

        float summe = 0;

        for ( final Eigenschaft eigenschaft : eigenschaften ) {

            summe += held.getStufe( eigenschaft );

        }

        return Math.round( summe / divisor );
    }

    /** @return die zum Berechnen der Formel noetigen Eigenschaften. */
    public Collection< Eigenschaft > getEigenschaften() {
        return new HashSet< Eigenschaft >( eigenschaften );
    }
}
