/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

public class VerbilligteSonderfertigkeitGelaendekunde implements
        VerbilligteSonderfertigkeit {

    public static final String BESCHREIBUNG = 
        "VerbilligteSonderfertigkeitGelaendekunde";
    
    public double getGpKosten() {
        return Sonderfertigkeit.DSCHUNGELKUNDIG.getApKosten() / 250.0;
    }

    /**
     * @throws UnsupportedOperationException wenn die Methode aufgerufen wird.
     */
    public final Sonderfertigkeit getSonderfertigkeit() {
        throw new UnsupportedOperationException(
                "Es stehen mehrere Gelaendekunden zur Auswahl" );
    }

    /**
     * @param held fuer den die verbilligte Gelaendekunde sein soll.
     * @return eine Liste alle Gelaendekunden die der Held noch nicht besitzt.
     */
    public final List< Sonderfertigkeit > getSonderfertigkeiten( final Held held ) {

        final List< Sonderfertigkeit > gelaendekunden;
        gelaendekunden = new ArrayList< Sonderfertigkeit >();

        for ( final Sonderfertigkeit gelaendekunde :
            Sonderfertigkeit.getGelaendekunden() ) {

            // Die Gelaendekunde kommt nur in die Liste wenn der Held sie nicht
            // besitzt und wenn sie nicht schon verbilligt ist.
            if ( ! held.hatVerbilligt(new SonderfertigkeitWert(gelaendekunde))
                    && ! held.hat( gelaendekunde ) ) {

                gelaendekunden.add( gelaendekunde );
            }
        }

        return gelaendekunden;
    }

    /** @return {@code true} */
    public final boolean hatAuswahl() {
        return true;
    }

    /**
     * @return eine Beschreibung welche Sonderfertigkeiten zur Auswahl stehen.
     */
    @Override
    public final String toString() {
        return I18n.i18n( BESCHREIBUNG );
    }

    /** @return das Tag mit dem Namen Gelaendekunde. */
    public final IXMLElement toXml() {
        return new XMLElement( HeldenbausteinXmlTags.TAG_GELAENDEKUNDE );
    }
}
