/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.util;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_BASISWERT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_MODIFIKATION;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_BASISWERT_MOD;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;

public class BasiswertModifikation {

    /** Der Basiswert der die Modifikation erhaelt. */
    private final Basiswert basiswert;

    /** Die Modifikation die vergeben wird. */
    private final int modifikation;

    /**
     * @param derBasiswert der die Modifikation erhaelt.
     * @param dieModifikation die auf den Basiswert vergeben wird.
     */
    public BasiswertModifikation(
            final Basiswert derBasiswert,
            final int dieModifikation ) {

        basiswert = derBasiswert;

        modifikation = dieModifikation;
    }

    /**
     * @return der Basiswert auf den der Bonus vergeben wird.
     */
    public final Basiswert getBasiswert() {
        return basiswert;
    }

    /**
     * @return die GP-Kosten die einem Baustein, durch diese Modifikation
     *         entstehen.
     */
    public double getGpKosten() {

        double kosten;

        if ( basiswert.equals( Basiswert.LE )
                || basiswert.equals( Basiswert.MR ) ) {

            kosten = 0.9;

        } else if ( basiswert.equals( Basiswert.AE ) ) {

            kosten = 0.6;

        } else if ( basiswert.equals( Basiswert.AU ) ) {

            kosten = 0.3;

        } else { //if ( basiswert.equals( Basiswert.INI ) ) {

            kosten = 0.5;

        }

        return kosten * modifikation;
    }

    /**
     * @return der Bonus auf den Basiswert.
     */
    public final int getModifikation() {
        return modifikation;
    }

    /**
     * @return etwas wie &quot;MR -4&quot; oder &quot;LE +2&quot;
     */
    @Override
    public final String toString() {
        
        return String.format(
                "%1$s %2$+d",
                basiswert.getKuerzel(),
                modifikation );
    }

    /**
     * @return ein XML-Element, das die Modifikation repraesentiert.
     */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_BASISWERT_MOD );

        element.setAttribute( ATTR_BASISWERT, basiswert.getId() );

        element.setAttribute( ATTR_MODIFIKATION,
                              String.valueOf( modifikation ) );

        return element;
    }
}
