/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.util;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_BASIS;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_W20;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_W6;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_KOERPERGROESSE;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Wuerfel;

public class GroesseFormel {

    /** Der Sockelbetrag zu dem die Wuerfelwuerfe addiert werden. */
    private final int basis;

    /** Die Anzahl der W20 die zum Sockelbetrag addiert werden. */
    private final int w20;

    /** Die Anzahl der W6 die zum Sockelbetrag addiert werden. */
    private final int w6;

    public GroesseFormel(
            final int derBasiswert,
            final int anzahlW20,
            final int anzahlW6 ) {

        basis = derBasiswert;

        w20 = anzahlW20;

        w6 = anzahlW6;
    }

    /** @return Der Basisbetrag zu dem die Wuerfelwuerfe addiert werden. */
    public final int getBasis() {
        return basis;
    }

    /** @return die Anzahl der W20 die gewuerfelt werden. */
    public final int getAnzahlW20() {
        return w20;
    }

    /** @return die Anzahl der W6 die gewuerfelt werden. */
    public final int getAnzahlW6() {
        return w6;
    }

    /**
     * 'Wuerfelt' eine Groesse aus.
     *
     * @return die zufaellig bestimmte Groesse.
     */
    public final int wuerfeln() {

        int groesse = basis;

        for ( int i = 0 ; i < w20 ; ++i ) {
            groesse += Wuerfel.W20.wuerfeln();
        }

        for ( int i = 0 ; i < w6 ; ++i ) {
            groesse += Wuerfel.W6.wuerfeln();
        }

        return groesse;
    }

    /**
     * @return die Formel als Text.
     */
    @Override
    public String toString() {

        final StringBuilder builder = new StringBuilder();

        builder.append( basis );
        
        if ( w20 == 1 ) {
        
            builder.append( "+" );
            builder.append( Wuerfel.W20.toString() );

        } else if ( w20 > 1 ) {
            
            builder.append( "+" );
            builder.append( w20 );
            builder.append( Wuerfel.W20.toString() );
        }
        
        if ( w6 == 1 ) {
            
            builder.append( "+" );
            builder.append( Wuerfel.W6.toString() );
        
        } else if ( w6 > 1 ) {
            
            builder.append( "+" );
            builder.append( w6 );
            builder.append( Wuerfel.W6.toString() );
        }
        
        return builder.toString();
    }

    /**
     * @return ein XML-Element, das die Formel repraesentiert.
     */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_KOERPERGROESSE );

        element.setAttribute( ATTR_BASIS, String.valueOf( basis ) );

        element.setAttribute( ATTR_W6, String.valueOf( w6 ) );

        element.setAttribute( ATTR_W20, String.valueOf( w20 ) );

        return element;
    }
}
