/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_EMPFOHLENE_VOR_NACH;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_VOR_NACHTEIL;

import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public class FactoryEmpfohleneVorNachteileOut {

    /**
     * Liefert die XML-Repraesentation der empfohlenen Vor-/Nachteile eines
     * Helden.
     *
     * @param baustein
     *            dessen empfohlene Vor-/Nachteile in XML umgewandelt werden
     *            sollen.
     * @return die Liste der empfohlenen Vor-/Nachteile oder {@code null}, wenn
     *         der Baustein keine empfohlenen Vor-/Nachteile hat.
     */
    public static IXMLElement erzeugen( final Heldenbaustein baustein ) {

        final List< VorNachteil > empfohlene =
            baustein.getEmpfohleneNachteile();

        empfohlene.addAll( baustein.getEmpfohleneVorteile() );

        if ( 0 == empfohlene.size() ) {

            return null;
        }

        final IXMLElement empfohleneElement = new XMLElement( TAG_EMPFOHLENE_VOR_NACH );

        for ( final VorNachteil vorNachteil : empfohlene ) {

            final XMLElement vorNachteilElement = new XMLElement( TAG_VOR_NACHTEIL );

            vorNachteilElement.setContent( vorNachteil.getId() );

            empfohleneElement.addChild( vorNachteilElement );
        }

        return empfohleneElement;
    }
}
