/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Locale;
import java.util.Vector;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeitgruppe;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.zauber.Zaubergruppe;

public class DialogEinstellungen extends JDialog implements Uebersetzbar {

    /** Der Text fuer den Button mit dem der Dialog geschlossen wird. */
    private static final String SCHLIESSEN = "Knopf.Schliessen";
    
    /** Die Taste die mit dem Schliessen-Knopf verbunden ist. */
    private static final String SCHLIESSEN_MNEMONIC = 
        "Knopf.Schliessen.Mnemonic";
    
    /**
     * Der Text vor der Combobox mit der die Sprache fuer das Programm gewaehlt
     * wird.
     */
    private static final String SPRACHE =
        "DialogEinstellungen.Sprache";

    /**
     * Der Titel fuer diesen Dialog. 
     */
    private static final String TITEL = 
        "DialogEinstellungen.Titel"; 

    /**
     * Hier muessen die Sprachen eingetragen werden, fuer die es eine 
     * Uebersetzung des Programms gibt.
     */
    private static final Locale[] LOCALES = {
        new Locale( "de" ),
        new Locale( "en" ),
        new Locale( "pl" )
    };
    
    /** Das Hauptpanel der Anwendung. */
    private Hauptpanel hauptpanel;
    
    /** Die Combobox mit der man die Sprache/Locale waehlt. */
    private final JComboBox spracheComboBox;
    
    private final Uebersetzer uebersetzer;
    
    public DialogEinstellungen( 
            final JFrame hauptfenster,
            final Hauptpanel dasHauptpanel ) {
        
        super( hauptfenster );
        
        hauptpanel = dasHauptpanel;
    
        uebersetzer = new Uebersetzer();
        
        final Einstellungen einstellungen = Einstellungen.getInstanz();
        
        final JLabel spracheLabel =  uebersetzer.erzeugeLabel( SPRACHE );
        
        spracheComboBox = erzeugeSpracheComboBox();
        
        final JCheckBox spracheCheckBox = new JCheckBox();
        spracheCheckBox.setSelected( einstellungen.isSpracheGesetzt() );
        spracheCheckBox.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                final Locale localeAlt = einstellungen.getSprache();

                einstellungen.setSpracheGesetzt( spracheCheckBox.isSelected() );

                spracheComboBox.setEnabled( spracheCheckBox.isSelected() );
                
                final Locale localeNeu = einstellungen.getSprache();
                
                if ( ! localeAlt.equals( localeNeu ) ) {

                    aendereSprache();
                }
            }
        });
        
        final JButton schliessen = uebersetzer.erzeugeButton( 
                SCHLIESSEN,
                SCHLIESSEN_MNEMONIC );
        schliessen.addActionListener( new ActionListener() {
           public void actionPerformed(ActionEvent e) {
               setVisible( false );
           } 
        });
        final JPanel buttonBar = ButtonBarFactory.buildCloseBar( schliessen );
        
        final FormLayout layout = new FormLayout( 
                "7dlu, P, 4dlu, P, 2dlu, P, 2dlu, P:G, 7dlu",
                "7dlu, P, 7dlu, P, 7dlu" );
        
        setLayout( layout );
        
        final CellConstraints cc = new CellConstraints();
        
        add( spracheLabel, cc.xy( 2, 2 ) );
        
        add( spracheCheckBox, cc.xy( 4, 2 ) );
        
        add( spracheComboBox, cc.xy( 6, 2 ) );
        
        add( buttonBar, cc.xyw( 2, 4, 7 ) );
        
        neuUebersetzen();
        
        pack();
    }
    
    /**
     * Aendert die Sprache des Programms, wenn der Benutzer eine neue Sprache
     * gewaehlt hat.
     */
    private void aendereSprache() {

        try {
            
            final Cursor waitCursor = new Cursor( Cursor.WAIT_CURSOR );
            
            setCursor( waitCursor );
            
            hauptpanel.setCursor( waitCursor );
        
            I18n.neuLaden();
                
            Sonderfertigkeitgruppe.neuSortieren();
            
            Talentgruppe.neuSortieren();
            
            Zaubergruppe.neuSortieren();
            
            hauptpanel.neuUebersetzen();
        
        } finally {
        
            final Cursor defaultCursor = new Cursor( Cursor.DEFAULT_CURSOR );
            
            setCursor( defaultCursor );
            
            hauptpanel.setCursor( defaultCursor );
        }
    }
    
    /**
     * Erzeugt die ComboBox mit den Sprachen fuer die das Programm lokalisiert
     * ist.
     * 
     * @return Die fertige ComboBox. Die aktuelle Sprache ist schon eingestellt. 
     */
    private JComboBox erzeugeSpracheComboBox() {
        
        final Einstellungen einstellungen = Einstellungen.getInstanz();
        
        final JComboBox comboBox = new JComboBox();
        
        comboBox.setEnabled( einstellungen.isSpracheGesetzt() );
        
        comboBox.setSelectedItem( 
                einstellungen.getSprache().getDisplayLanguage() );
        
        comboBox.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int index = comboBox.getSelectedIndex();
                
                final Locale localeAlt = einstellungen.getSprache();
                
                einstellungen.setSprache( LOCALES[ index ] );
                
                final Locale localeNeu = einstellungen.getSprache();
                
                if ( ( ! localeAlt.equals( localeNeu ) ) 
                        && einstellungen.isSpracheGesetzt() ) {

                    aendereSprache();
                }
            }
        });
        
        return comboBox;
    }

    /**
     * Aktualisiert die angezeigten Texte, nachdem die Sprache geaendert wurde.
     */
    public final void neuUebersetzen() {

        setTitle( I18n.i18n( TITEL ) );
        
        final Vector< String > sprachen = new Vector< String >( LOCALES.length );
        
        final Einstellungen einstellungen = Einstellungen.getInstanz();
        final Locale sprache = einstellungen.getSprache();
        
        for ( Locale locale : LOCALES ) {
            
            sprachen.add( locale.getDisplayLanguage( sprache ) );
        }
        
        final ComboBoxModel model = new DefaultComboBoxModel( sprachen );
        
        spracheComboBox.setModel( model );
        spracheComboBox.setSelectedItem( sprache.getDisplayLanguage( sprache ) );
        
        uebersetzer.neuUebersetzen();

        pack();
    }
}
