/*
  * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;

import de.bernhardjung.dsaprogramm.Debug;
import de.bernhardjung.dsaprogramm.gui.action.ActionBeenden;
import de.bernhardjung.dsaprogramm.gui.action.ActionDatenquelle;
import de.bernhardjung.dsaprogramm.gui.action.ActionEinstellungen;
import de.bernhardjung.dsaprogramm.gui.action.ActionFernwaffeneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionHilfe;
import de.bernhardjung.dsaprogramm.gui.action.ActionInfo;
import de.bernhardjung.dsaprogramm.gui.action.ActionKultureneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionLadenHeldXml;
import de.bernhardjung.dsaprogramm.gui.action.ActionLookAndFeel;
import de.bernhardjung.dsaprogramm.gui.action.ActionNahkampfwaffeneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionNameFactoryseditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionNamenlisteneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionNeuerHeld;
import de.bernhardjung.dsaprogramm.gui.action.ActionProfessioneneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionRasseneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionRuestungeneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionSchildeeditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernHeldRtf;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernHeldXml;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaFernkampfwaffen;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaKultur;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaNahkampfwaffen;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaNameFactorys;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaNamenlisten;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaProfession;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaRasse;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaRuestungen;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaSchilde;
import de.bernhardjung.dsaprogramm.gui.steigern.PanelSteigern;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.LookAndFeelAktualisierbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class Hauptpanel 
extends JPanel 
implements Uebersetzbar, LookAndFeelAktualisierbar {
    
    /**
     * Die Action um das Programm zu beenden.
     */
    private final ActionBeenden actionBeenden;

    /** Die Action die den Dialog zu Auswaehlen der Datenquelle aufruft. */
    private final ActionDatenquelle actionDatenquelle;

    /** Zeigt den Einstellungsdialog an. */
    private final ActionEinstellungen actionEinstellungen;
    
    /** Ruft den Fernwaffeneditor auf. */
    private final ActionFernwaffeneditor actionFernwaffeneditor;
    
    /** Zeigt die Hilfe an. */
    private final ActionHilfe actionHilfe;
    
    /** Zeigt den Info-Dialog mit den Lizenzen und dem Disclaimer an. */
    private final ActionInfo actionInfo;
    
    /** Zeigt den Kultureneditor an. */
    private final ActionKultureneditor actionKultureneditor;
    
    /** Die Action um den Held aus einer XML-Datei zu laden. */
    private final ActionLadenHeldXml actionLadenXml;

    /** Zeigt den Nahkampfwaffeneditor an. */
    private final ActionNahkampfwaffeneditor actionNahkampfwaffeneditor;
    
    /** Zeigt den Editor fuer die Namenfactory an. */
    private final ActionNameFactoryseditor actionNameFactoryseditor;
    
    /** Zeigt den Editor fuer die Namenlisten an. */
    private final ActionNamenlisteneditor actionNamenlisteneditor;
    
    /** Startet die Heldenerschaffung */
    private final ActionNeuerHeld actionNeuerHeld;
    
    /** Zeigt den Editor fuer die Professionen an. */
    private final ActionProfessioneneditor actionProfessioneneditor;
    
    /** Die Action die den Rasseneditor anzeigt. */
    private final ActionRasseneditor actionRasseneditor;
    
    /** Zeigt den Editor fuer die Ruestungen an. */
    private final ActionRuestungeneditor actionRuestungeneditor;
    
    /** Zeigt den Editor fuer die Schilde an. */
    private final ActionSchildeeditor actionSchildeeditor;
    
    /** Die Action um den Held als RTF-Datei zu speichern. */
    private final ActionSpeichernHeldRtf actionSpeichernRtf;

    /** Die Action um den Held als XML-Datei zu speichern. */
    private final ActionSpeichernHeldXml actionSpeichernXml;

    /**
     * Das Panel das aktuell im Fenster angzeigt wird. eigentlich liefert 
     * dieses Objekt ueber seine getPanel-Methode das anzuzeigende Panel.
     * Das gelieferte Panel wird dann die {@link #hauptcomponent}. 
     */
    private WechselbaresPanel aktuellesPanel;
    
    /** Der (Internal)Frame, der das Panel enthaelt. */
    private final Container elternkomponente;

    /** Die JComponent die in der Mitte des Hauptfensters angezeigt wird. */
    private JComponent hauptcomponent;

    /** Das Hauptfenster der Anwendung. */
    private final JFrame hauptfenster;

    /** 
     * Der Schriftzug der groß im Fenster angezeigt wird, wenn man das 
     * Programm startet.
     */
    private final JLabel logo;
    
    /** Das Menue der Anwendung. */
    private final JMenuBar menue;
    
    /** Das Menue Aktionen. */
    private final JMenu aktionen;
    
    /** Das Menue Daten. */
    private final JMenu daten;
    
    /** Das Panel mit dem ein Held erschaffen wird. */
    private PanelErschaffung panelErschaffung;

    /** Das Panel mit dem man die Fernwaffen bearbeiten kann. */
    private PanelMetaFernkampfwaffen panelMetaFernwaffen;

    /** Das Panel mit dem man einen Kultur eingeben kann. */
    private PanelMetaKultur panelMetaKultur;

    /** Das Panel mit dem man die Nahkampfwaffen bearbeiten kann. */
    private PanelMetaNahkampfwaffen panelMetaNahkampfwaffen;

    /** Das Panel mit dem man die Namensfactories bearbeiten kann. */
    private PanelMetaNameFactorys panelMetaNameFactories;

    /** Der Editor in den man die Namenlisten eingeben kann. */
    private PanelMetaNamenlisten panelMetaNamenlisten;
    
    /** Das Panel mit dem man einen Profession eingeben kann. */
    private PanelMetaProfession panelMetaProfession;

    /** Das Panel mit dem man einen Rasse eingeben kann. */
    private PanelMetaRasse panelMetaRasse;

    /** Der Editor fuer die Ruestungen. */
    private PanelMetaRuestungen panelMetaRuestungen;

    /** Der Editor fuer die Schilde. */
    private PanelMetaSchilde panelMetaSchilde;

    /** Das Panel mit dem ein Held gesteigert wird. */
    private PanelSteigern panelSteigern;

    /**
     * 
     * @param dasHauptfenster
     * @param dieElternkomponente
     * @param alleinstehend
     *            {@code true}, wenn das Programm als alleinstehende Anwendung
     *            laeuft. {@code false}, wenn das Programm als Plugin im DSA
     *            Desktop laeuft.
     */
    public Hauptpanel(
            final JFrame dasHauptfenster,
            final Container dieElternkomponente,
            final boolean alleinstehend ) {

        super( new BorderLayout() );
        
        hauptfenster = dasHauptfenster;

        elternkomponente = dieElternkomponente;

        actionBeenden = new ActionBeenden( this );

        actionDatenquelle = new ActionDatenquelle( this );

        actionEinstellungen = new ActionEinstellungen( this );
        
        actionFernwaffeneditor = new ActionFernwaffeneditor( this );
        
        actionHilfe = new ActionHilfe( this );
        
        actionInfo = new ActionInfo( this );
     
        actionKultureneditor = new ActionKultureneditor( this );
        
        actionNahkampfwaffeneditor = new ActionNahkampfwaffeneditor( this );
        
        actionNameFactoryseditor = new ActionNameFactoryseditor( this );
        
        actionNamenlisteneditor = new ActionNamenlisteneditor( this );
        
        actionNeuerHeld = new ActionNeuerHeld( this ) ;
        
        actionProfessioneneditor = new ActionProfessioneneditor( this );
        
        actionRasseneditor = new ActionRasseneditor( this );
        
        actionRuestungeneditor = new ActionRuestungeneditor( this );
        
        actionSchildeeditor = new ActionSchildeeditor( this );
        
        actionSpeichernRtf = new ActionSpeichernHeldRtf( this );
        actionSpeichernRtf.setEnabled( false );

        actionLadenXml = new ActionLadenHeldXml( this );

        actionSpeichernXml =
            new ActionSpeichernHeldXml( this );
        actionSpeichernXml.setEnabled( false );

        aktionen = new JMenu();
        
        aktuellesPanel = null;
        
        daten = new JMenu();
        
        menue = erzeugeMenue( alleinstehend );
        
        logo = new JLabel( Util.ladeIcon( I18n.i18n( "Hauptpanel.Logo" ) ) );
        anzeigenComponent( logo );

        elternkomponente.setSize( 640, 480 );
    }

    /**
     * Aktualisiert das Aussehn des Programms nachdem das Look & Feel geaendert
     * wurde.
     */
    public final void aktualisiereLookAndFeel() {
        
        SwingUtilities.updateComponentTreeUI( this );
        
        SwingUtilities.updateComponentTreeUI( menue );
        
        final JComponent[] components = {
                panelErschaffung,
                panelMetaFernwaffen,
                panelMetaKultur,
                panelMetaNahkampfwaffen,
                panelMetaNameFactories,
                panelMetaNamenlisten,
                panelMetaProfession,
                panelMetaRasse,
                panelMetaRuestungen,
                panelMetaSchilde,
                panelSteigern
        };
        
        for ( JComponent component : components ) {
            
            if ( null != component ) {
                
                SwingUtilities.updateComponentTreeUI( component );
            }
        }
        
        Util.aktualisiereLookAndFeel( 
                actionLadenXml, 
                actionSpeichernRtf,
                actionSpeichernXml,
                panelMetaKultur,
                panelMetaProfession,
                panelMetaRasse );
    }
    
    /**
     * Maximiert das Hauptfenster so weit, dass sein Inhalt ganz angezeigt wird.
     */
    public final void anpassenGroesse() {

        final Dimension preferred = elternkomponente.getPreferredSize();

        final Dimension size = elternkomponente.getSize();

        if ( preferred.height > size.height ) {
            size.height = preferred.height;
        }

        if ( preferred.width > size.width ) {
            size.width = preferred.width;
        }

        elternkomponente.setSize( size );
    }

    /**
     * Bestimmt was im Hauptfenster anzeigt wird.
     * 
     * @param panel Das anzuzeigende Panel.
     * @return {@code true}, wenn das Panel angezeigt werden kann.
     */
    public final void anzeigen( final WechselbaresPanel panel ) {
        
        if ( null == aktuellesPanel || panel != aktuellesPanel ) {
            
            aktuellesPanel = panel;
            
            aktuellesPanel.reset();
            
            anzeigenComponent( panel );
        }
    }
    
    /**
     * Die eigentliche Methode die bestimmt was im Hauptfenster angezeigt wird.
     *
     * @param component die dargestellt wird.
     */
    private void anzeigenComponent( final JComponent component ) {

        if ( null != component
         && component != hauptcomponent ) {
            
            if ( null != hauptcomponent ) {

                remove( hauptcomponent );

            }

            if ( component.equals( panelSteigern ) ) {

                actionSpeichernXml.setEnabled( true );
                actionSpeichernRtf.setEnabled( true );

            } else {

                actionSpeichernXml.setEnabled( false );
                actionSpeichernRtf.setEnabled( false );

            }

            hauptcomponent = component;

            add( hauptcomponent, BorderLayout.CENTER );

            anpassenGroesse();

            validate();
            repaint();
        }
    }
    
    private void debugAllePanelErzeugen() {

        try {
        
            zeigeWartenMauszeiger();
            
            actionDatenquelle.getDialogDatenquelle();
            
            actionEinstellungen.getDialogEinstellungen();
            
            actionHilfe.getHelpBroker();
            
            actionInfo.getDialogInfo();
            
            getPanelErschaffung().erzeugeAllePanel( this );
            
            getPanelMetaFernwaffen();
            
            getPanelMetaKultur();
            
            getPanelMetaNahkampfwaffen();
            
            getPanelMetaNameFactories();
            
            getPanelMetaNamenlisten();
            
            getPanelMetaProfession();
            
            getPanelMetaRasse();
            
            getPanelMetaRuestungen();
            
            getPanelMetaSchilde();
            
            getPanelSteigern().debugSetHeld( this );
        
        } finally {
            
            zeigeStandardMauszeiger();
        }
    }

    private void debugAllePanelUebersetzen() {

        try {
        
            zeigeWartenMauszeiger();
            
            actionDatenquelle.getDialogDatenquelle().neuUebersetzen();
            
            actionEinstellungen.getDialogEinstellungen().neuUebersetzen();
            
            actionHilfe.getHelpBroker();
            actionHilfe.neuUebersetzen();
            
            actionInfo.getDialogInfo().neuUebersetzen();
            
            getPanelErschaffung().erzeugeAllePanel( this );
            getPanelErschaffung().neuUebersetzen();
            
            getPanelMetaFernwaffen().neuUebersetzen();
            
            getPanelMetaKultur().neuUebersetzen();
            
            getPanelMetaNahkampfwaffen().neuUebersetzen();
            
            getPanelMetaNameFactories().neuUebersetzen();
            
            getPanelMetaNamenlisten().neuUebersetzen();
            
            getPanelMetaProfession().neuUebersetzen();
            
            getPanelMetaRasse().neuUebersetzen();
            
            getPanelMetaRuestungen().neuUebersetzen();
            
            getPanelMetaSchilde().neuUebersetzen();
            
            getPanelSteigern().debugSetHeld( this ).neuUebersetzen();
        
        } finally {
            
            zeigeStandardMauszeiger();
        }
    }
    
    private JMenuBar erzeugeMenue( boolean alleinstehend ) {

        JMenuBar menue = new JMenuBar();

        aktionen.setText( I18n.i18n( "Hauptpanel.Menue.Aktionen" ) );
        menue.add( aktionen );
        String mnemonic =
            I18n.i18n( "Hauptpanel.Menue.Aktionen.Mnemonic" );
        aktionen.setMnemonic( Util.getKeyCode( mnemonic ) );

        aktionen.add( actionNeuerHeld );

        aktionen.add( actionSpeichernRtf );

        aktionen.add( actionLadenXml );

        aktionen.add( actionSpeichernXml );

        aktionen.addSeparator();

        aktionen.add( actionHilfe );

        aktionen.add( actionInfo );

        aktionen.addSeparator();

        aktionen.add( actionEinstellungen );

        if ( alleinstehend ) {

            aktionen.addSeparator();

            aktionen.add( actionBeenden );
        }

        daten.setText( I18n.i18n( "Hauptpanel.Menue.Daten" ) );
        menue.add( daten );
        mnemonic = I18n.i18n( "Hauptpanel.Menue.Daten.Mnemonic" );
        daten.setMnemonic( Util.getKeyCode( mnemonic ) );

        daten.add( actionDatenquelle );

        daten.add( actionRasseneditor );

        daten.add( actionKultureneditor );

        daten.add( actionProfessioneneditor );

        daten.add( actionRuestungeneditor );

        daten.add( actionNahkampfwaffeneditor );

        daten.add( actionFernwaffeneditor );

        daten.add( actionSchildeeditor );

        daten.add( actionNamenlisteneditor );

        daten.add( actionNameFactoryseditor );
        
        if ( alleinstehend )
        {
            final JMenu lookAndFeel = new JMenu( 
                    I18n.i18n( "Hauptpanel.Menue.LookAndFeel" ) );
            menue.add( lookAndFeel );
            
            mnemonic = I18n.i18n( "Hauptpanel.Menue.LookAndFeel.Mnemonic" );
            lookAndFeel.setMnemonic( Util.getKeyCode( mnemonic ) );
            
            ButtonGroup buttonGroup = new ButtonGroup();
            
            final String aktuellesLookAndFeel = 
                Einstellungen.getInstanz().getLookAndFeel();
            
            for ( final LookAndFeelInfo laf : UIManager.getInstalledLookAndFeels() ) {
                
                final Action action = new ActionLookAndFeel( laf, this );
                
                final JRadioButtonMenuItem menuItem;
            
                menuItem = new JRadioButtonMenuItem( action );
                
                final String lafKlasse = laf.getClassName();
                
                menuItem.setSelected( lafKlasse.equals( aktuellesLookAndFeel ));
                
                buttonGroup.add( menuItem );
                
                lookAndFeel.add( menuItem );
            }
        }
        
        if ( Debug.DEBUG ) {
            
            final JMenu debug = new JMenu( "Debug" );
            menue.add( debug );
            
            debug.add( new AbstractAction( "Alle Panel erzeugen" ) {
                public void actionPerformed( ActionEvent e ) {
                    debugAllePanelErzeugen();
                } } );
            
            debug.add( new AbstractAction( "Alle Panel uebersetzen" ) {
                public void actionPerformed( ActionEvent e ) {
                    debugAllePanelUebersetzen();
                } } );
            
            final JMenu panel = new JMenu( "Panel" );
            debug.add( panel );

            panel.add( new AbstractAction( "Erschaffung/Leitzauber") {
                public void actionPerformed( ActionEvent e ) {
                    getPanelErschaffung().debugZeigePanelLeitzauber();
                    anzeigenComponent( (JComponent) getPanelErschaffung() );
                } } );
            
            panel.add( new AbstractAction( "Erschaffung/RasKulProf" ) {
                public void actionPerformed( ActionEvent e ) {
                    getPanelErschaffung().debugZeigePanelRasKulProf();
                    anzeigenComponent( (JComponent) getPanelErschaffung() );
                } } );

            panel.add( new AbstractAction( "Erschaffung/Sonderfertigkeit") {
                public void actionPerformed( ActionEvent e ) {
                    getPanelErschaffung().debugZeigePanelSonderfertigkeitWert();
                    anzeigenComponent( (JComponent) getPanelErschaffung() );
                } } );

            panel.add( new AbstractAction( "Erschaffung/Sprache") {
                public void actionPerformed( ActionEvent e ) {
                    getPanelErschaffung().debugZeigePanelSprache();
                    anzeigenComponent( (JComponent) getPanelErschaffung() );
                } } );

            panel.add( new AbstractAction( "Erschaffung/Steigern") {
                public void actionPerformed( ActionEvent e ) {
                    getPanelErschaffung().debugZeigePanelSteigern( 
                            Hauptpanel.this );
                    anzeigenComponent( (JComponent) getPanelErschaffung() );
                } } );

            panel.add( new AbstractAction( "Erschaffung/TalentbonusEntdecker") {
                public void actionPerformed( ActionEvent e ) {
                    getPanelErschaffung().debugZeigePanelTalentBonusEntdecker();
                    anzeigenComponent( (JComponent) getPanelErschaffung() );
                } } );

            panel.add( new AbstractAction( "Erschaffung/TalentbonusFest") {
                public void actionPerformed( ActionEvent e ) {
                    getPanelErschaffung().debugZeigePanelTalentBonusFest();
                    anzeigenComponent( (JComponent) getPanelErschaffung() );
                } } );

            panel.add( new AbstractAction( "Erschaffung/TalentbonusFrei") {
                public void actionPerformed( ActionEvent e ) {
                    getPanelErschaffung().debugZeigePanelTalentBonusFrei();
                    anzeigenComponent( (JComponent) getPanelErschaffung() );
                } } );

            panel.add( new AbstractAction( "Erschaffung/VorNachteilWert") {
                public void actionPerformed( ActionEvent e ) {
                    getPanelErschaffung().debugZeigePanelVorNachteilWert();
                    anzeigenComponent( (JComponent) getPanelErschaffung() );
                } } );
        }

        return menue;
    }
    
    /**
     * Prueft ob das aktuell im Hauptpanel angezeigte Panel gewechselt werden
     * kann. Wenn das aktuelle Panel noch ungespeicherte Aenderungen enthaelt
     * hat es hier die Gelegenheit den Benutzer darauf aufmerksam zu machen und
     * zu fragen, ob er wirklich die aktuell laufende Aktion abbrechen will.
     * 
     * @return {@code true} wenn das aktuelle Panel gewechselt werden kann,
     *         {@code false}, falls der Benutzer dem Wechsel zugestimmt hat.
     */
    public final boolean kannPanelWechseln() {
        
        return null == aktuellesPanel || aktuellesPanel.kannGewechseltWerden();
    }

    public void neuUebersetzen() {
        
        Uebersetzer.neuUebersetzen(
                actionBeenden,
                actionDatenquelle,
                actionEinstellungen,
                actionFernwaffeneditor,
                actionHilfe,
                actionInfo,
                actionKultureneditor,
                actionLadenXml,
                actionNahkampfwaffeneditor,
                actionNameFactoryseditor,
                actionNamenlisteneditor,
                actionNeuerHeld,
                actionProfessioneneditor,
                actionRasseneditor,
                actionRuestungeneditor,
                actionSchildeeditor,
                actionSpeichernRtf,
                actionSpeichernXml,
                panelErschaffung,
                panelMetaFernwaffen,
                panelMetaKultur,
                panelMetaNahkampfwaffen,
                panelMetaNameFactories,
                panelMetaNamenlisten,
                panelMetaProfession,
                panelMetaRasse,
                panelMetaRuestungen,
                panelMetaSchilde,
                panelSteigern
        );
        
        hauptfenster.setTitle( I18n.i18n( "Programm.Name" ) );
        
        String mnemonic;
        
        aktionen.setText( I18n.i18n( "Hauptpanel.Menue.Aktionen" ) );
        mnemonic = I18n.i18n( "Hauptpanel.Menue.Aktionen.Mnemonic" );
        aktionen.setMnemonic( Util.getKeyCode( mnemonic ) );

        daten.setText( I18n.i18n( "Hauptpanel.Menue.Daten" ) );
        mnemonic = I18n.i18n( "Hauptpanel.Menue.Daten.Mnemonic" );
        daten.setMnemonic( Util.getKeyCode( mnemonic ) );
        
        if ( logo == hauptcomponent ) {
            
            logo.setIcon( Util.ladeIcon( I18n.i18n( "Hauptpanel.Logo" ) ) );
        }
    }
    
    /**
     * @return das Menue der Anwendung.
     */
    public final JMenuBar getMenue() {
        
        return menue;
    }

    /**
     * @return die Action um das Programm zu beenden, wenn es als alleinstehende
     *         Anwendung laeuft.
     */
    public final Action getActionBeenden() {
        return actionBeenden;
    }

    /**
     * @return Das Hauptfenster der Anwendung.
     */
    public final JFrame getHauptfenster() {
        
        return hauptfenster;
    }
    
    /**
     * @return das Panel, das zum Erschaffen von Helden verwendet wird.
     */
    public final PanelErschaffung getPanelErschaffung() {
        if ( null == panelErschaffung ) {
            panelErschaffung = new PanelErschaffung();
        }
        return panelErschaffung;
    }

    /***
     * @return das Panel  mit dem man die Fernwaffen bearbeiten kann.
     */
    public final PanelMetaFernkampfwaffen getPanelMetaFernwaffen() {
        if ( null == panelMetaFernwaffen ) {
            panelMetaFernwaffen = new PanelMetaFernkampfwaffen();
        }
        return panelMetaFernwaffen;
    }

    /**
     * @return das Panel, mit dem man eine Kultur eingeben kann.
     */
    public final PanelMetaKultur getPanelMetaKultur() {
        if ( null == panelMetaKultur ) {
            panelMetaKultur = new PanelMetaKultur( hauptfenster, this );
        }
        return panelMetaKultur;
    }

    /**
     * @return das Panel  mit dem man die Nahkampfwaffen bearbeiten kann.
     */
    public final PanelMetaNahkampfwaffen getPanelMetaNahkampfwaffen() {
        if ( null == panelMetaNahkampfwaffen ) {
            panelMetaNahkampfwaffen = new PanelMetaNahkampfwaffen();
        }
        return panelMetaNahkampfwaffen;
    }

    /**
     * @return das Panel mit dem man die Namensfactories bearbeiten kann.
     */
    public final PanelMetaNameFactorys getPanelMetaNameFactories() {
        if ( null == panelMetaNameFactories ) {
            panelMetaNameFactories = new PanelMetaNameFactorys();
        }
        return panelMetaNameFactories;
    }

    /**
     * @return Der Editor mit dem man die Namenlisten bearbeiten kann.
     */
    public final PanelMetaNamenlisten getPanelMetaNamenlisten() {
        if ( null == panelMetaNamenlisten ) {
            panelMetaNamenlisten = new PanelMetaNamenlisten();
        }
        return panelMetaNamenlisten;
    }

    /**
     * @return das Panel, mit dem man eine Profession eingeben kann.
     */
    public final PanelMetaProfession getPanelMetaProfession() {
        if ( null == panelMetaProfession ) {
            panelMetaProfession = new PanelMetaProfession( hauptfenster, this );
        }
        return panelMetaProfession;
    }

    /**
     * @return das Panel, mit dem man eine Rasse eingeben kann.
     */
    public final PanelMetaRasse getPanelMetaRasse() {
        if ( null == panelMetaRasse ) {
            panelMetaRasse = new PanelMetaRasse( hauptfenster, this );
        }
        return panelMetaRasse;
    }

    /**
     * @return das Panel, mit dem man die Ruestungen editieren kann.
     */
    public final PanelMetaRuestungen getPanelMetaRuestungen() {
        if ( null == panelMetaRuestungen ) {
            panelMetaRuestungen = new PanelMetaRuestungen();
        }
        return panelMetaRuestungen;
    }

    /**
     * @return den Editor fuer die Schilde.
     */
    public final PanelMetaSchilde getPanelMetaSchilde() {
        if ( null == panelMetaSchilde ) {
            panelMetaSchilde = new PanelMetaSchilde();
        }
        return panelMetaSchilde;
    }

    /**
     * @return das Panel das zum Steigern von Helden verwendet wird.
     */
    public final PanelSteigern getPanelSteigern() {
        if ( null == panelSteigern ) {
            panelSteigern = new PanelSteigern( actionSpeichernXml );
        }
        return panelSteigern;
    }
    
    /**
     * Ruft den Dialog zu Auswaehlen der Datenquelle auf.
     */
    public final void zeigeDialogDatenquelle() {
        actionDatenquelle.actionPerformed( null );
    }
    
    /**
     * Zeigt wieder den Standard-Mauszeiger an, wenn er vorher auf den 
     * Warten-Zeiger gestellt wurde.
     */
    public final void zeigeStandardMauszeiger() {
        
        hauptfenster.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
    }
    
    /**
     * Zeigt den Warten-Mauszeiger ueber dem Hauptfenster an.
     */
    public final void zeigeWartenMauszeiger() {
        
        hauptfenster.setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
    }
}
