/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import static de.bernhardjung.dsaprogramm.gui.PanelTalenteListe.AP;
import static de.bernhardjung.dsaprogramm.gui.PanelTalenteListe.FAKTOR;
import static de.bernhardjung.dsaprogramm.gui.PanelTalenteListe.ICON_LEITTALENT;
import static de.bernhardjung.dsaprogramm.gui.PanelTalenteListe.KOSTEN;
import static de.bernhardjung.dsaprogramm.gui.PanelTalenteListe.SE;
import static de.bernhardjung.dsaprogramm.gui.PanelTalenteListe.TALENT;
import static de.bernhardjung.dsaprogramm.gui.PanelTalenteListe.TAW;
import static de.bernhardjung.dsaprogramm.gui.PanelTalenteListe.TEXT_LEITTALENT;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelKampftechnikenListe
extends ResetbaresPanel
implements Observer, Uebersetzbar {

    /** Die Ueberschrift fuer die Spalte mit den AT-Punkten. */
    private static final String AT = "PanelKampftechniken.AT";

    /** Die Ueberschrift fuer die Spalte mit den PA-Punkten. */
    private static final String PA = "PanelKampftechniken.PA";

    /**
     * Die Map enthaelt Regelelemente (Talente, Eigenschaften, ...) als
     * Schluessel.
     * Die Listen speichern Indizes von Talenten in {@link #talente}.
     * Die Talente in der Liste sind von dem entsprechenden Regelelement
     * abhaengig.
     */
    private final Map< Object, List< Integer > > abhaengigkeiten;
    
    /**
     * Die Spinner in denen eingestellt wird wie viel Punkte der Stufe
     * eines Talents auf Attacke gelegt werden.
     */
    private final List< JSpinner > attacken;

    /**
     * Die JCheckBoxen die in der ersten Spalte angezeigt werden.
     * Die i-te Checkbox in der Oberflaeche (von oben gezaehlt) steht an
     * Listen-Position i.
     */
    private final List< JCheckBox > checkBoxes;

    /** Eine fette Schriftart. */
    private final Font fetteSchrift;

    /** Der Held dessen Wert angezeigt und manipuliert werden. */
    private Held held;

    /**
     * Die JLabel mit denen die Steigerungskosten angezeigt werden.
     * Das i-te Label in der Oberflaeche (von oben gezaehlt) steht an
     * Listen-Position i.
     */
    private final List< JLabel > kosten;
    
    /**
     * Die Icons die ein Leittalent markieren.
     */
    private final List< JLabel > leittalente;

    /** Das Objekt mit dem beim Steigern die Lernmethode festgelegt wird. */
    private final Lernmethoden lernmethoden;

    /**
     * Das Panel auf dem die ganzen Elemente der Benutzeroberflaeche angeordnet
     * werden. Es wird in eine Scrollpane verpackt und auf dies 
     */
    private final JPanel panel;
    
    /**
     * Die JLabel welche die Anzahl an Punkten darstellen, die auf
     * die Parade verteilt wurden.
     */
    private final List< JLabel > paraden;

    
    
    /** Die Checkboxen mit denen man eine spezielle Erfahrung vergeben kann. */
    private final List< JCheckBox > spezielleErfahrungen;

    /**
     * {@code true}, wenn dass Panel zum Steigern der Talente verwendet wird,
     * {@code false}, wenn es bei der Charaktererschaffung verwendet wird.
     */
    private final boolean steigern;

    /**
     * Die JLabel die die Steigerungsfaktoren der Talente anzeigen.
     * Das i-te Label in der Oberflaeche (von oben gezaehlt) steht an
     * Listen-Position i.
     */
    private final List< JLabel > steigerungsfaktoren;

    /**
     * Die JSpinner mit denen die Stufen der Talente angezeigt und manipuliert
     * werden.
     * Der Spinner an Listen-Position i steht in der Oberflaeche an i-ter
     * Stelle (von oben gezaehlt).
     */
    private final List< JSpinner > stufen;

    /**
     * Die angezeigten Talente.
     * Das Talent an Position i wird in der Benutzeroberflaech in Zeile i
     * angezeigt. Die Zeilen mit den Trennstrichen nicht mitgerechnet.
     */
    private final Talentgruppe talente;
    
    /** Uebersetzt diverse Label des Panels. */
    private final Uebersetzer uebersetzer;
    
    /**
     * Erzeugt ein neues Panel.
     *
     * @param zumSteigern
     *            {@code true}, wenn dass Panel zum Steigern der Talente
     *            verwendet wird, {@code false}, wenn es bei der
     *            Charaktererschaffung verwendet wird.
     * @param dieLernmethoden
     *          Gibt an welche Lernmethode zum Steigern verwendet werden soll.
     */
    public PanelKampftechnikenListe(
            final boolean zumSteigern,
            final Lernmethoden dieLernmethoden ) {

        super();

        uebersetzer = new Uebersetzer();
        
        
        
        held = new Held();

        talente = Talentgruppe.KAMPF;

        abhaengigkeiten = new HashMap< Object, List< Integer > >();

        initAbhaengigkeiten();

        attacken = new ArrayList< JSpinner >( talente.size() );

        checkBoxes = new ArrayList< JCheckBox >( talente.size() );

        fetteSchrift = getFont().deriveFont( Font.BOLD );

        kosten = new ArrayList< JLabel >( talente.size() );

        lernmethoden = dieLernmethoden;
        if ( null != lernmethoden ) {
            lernmethoden.addObserver( this );
        }

        leittalente = new ArrayList< JLabel >( talente.size() );

        panel = new JPanel();
        
        paraden = new ArrayList< JLabel >( talente.size() );
        
        spezielleErfahrungen = new ArrayList< JCheckBox >( talente.size() );
        
        steigern = zumSteigern;

        steigerungsfaktoren = new ArrayList< JLabel >( talente.size() );

        stufen = new ArrayList< JSpinner >( talente.size() );
        
        aufbauenBenutzeroberflaeche();
    }

    /**
     * Baut die Benutzeroberflaeche auf.
     */
    private void aufbauenBenutzeroberflaeche() {
     
        final ActionListener actionDeAktivieren;
        final ChangeListener actionStufe;

        if ( steigern ) {

            actionDeAktivieren = new ActionDeAktivierenSteigern();

            actionStufe = new ActionStufeSteigern();

        } else {

            actionDeAktivieren = new ActionDeAktivierenErschaffung();

            actionStufe = new ActionStufeErschaffung();
        }

        final ActionListener actionSpezielleErfahrung = new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                final JCheckBox checkBox = (JCheckBox) e.getSource();

                final int index = Integer.valueOf( checkBox.getName() );

                final Talent talent = talente.get( index );

                if ( held.hat( talent ) ) {

                    final TalentWert wert = held.getWert( talent );

                    wert.setSpezielleErfahrung( checkBox.isSelected() );
                }
            }
        };

        panel.setLayout( erzeugeLayout() );

        final CellConstraints cc = new CellConstraints();

        int x = 2;
        
        panel.add( 
                uebersetzer.erzeugeUeberschrift( TALENT ), cc.xy( x += 2, 2 ) );
        
        panel.add( 
                uebersetzer.erzeugeUeberschrift( FAKTOR ), 
                cc.xy( x += 2, 2 ) );
        
        panel.add( uebersetzer.erzeugeUeberschrift( AT ), cc.xy( x += 2, 2 ) );
        
        panel.add( uebersetzer.erzeugeUeberschrift( PA ), cc.xy( x += 2, 2 ) );
        
        panel.add( uebersetzer.erzeugeUeberschrift( TAW ), cc.xy( x += 2, 2 ) );
        
        panel.add( 
                uebersetzer.erzeugeUeberschrift( KOSTEN ), 
                cc.xy( x += 2, 2 ) );
        
        if ( steigern ) {
            
            panel.add( 
                    uebersetzer.erzeugeUeberschrift( SE ), 
                    cc.xy( x += 2, 2 ) );
        }

        // In der 2. und ab da in jeder 5. Zeile wird ein Trennstrich
        // eingefuegt.
        final int extraZeilen = 1 + ( talente.size() / 4 );
        // trennstriche gibt an wie viele Trennstriche schon eingefuegt wurden.
        final ChangeListener attackeListener = new AttackeListener();

        for ( int i = 0, n = talente.size() + extraZeilen, trennstriche = 0
                ; i < n
                ; ++i ) {

            x = 2;
            final int y = ( 2 * i ) + 4;

            // alle 5 Zeilen ein Trennstrich.
            if ( ( i % 5 ) == 0 ) {

                panel.add( new JSeparator(), cc.xyw( x, y, 16 ) );

                ++trennstriche;

                continue;
            }

            // Die Markierung fuer Leittalente
            // *******************************
            final JLabel leittalent = new JLabel();

            leittalente.add( leittalent );

            panel.add( leittalent, cc.xy( x, y ) );

            // Die Checkbox zum Aktivieren
            // ***************************
            final Kampftechnik talent;
            talent = (Kampftechnik) talente.get( i - trennstriche );

            final JCheckBox checkBox = new JCheckBox( talent.getName() );

            checkBox.addActionListener( actionDeAktivieren );

            checkBox.setName( String.valueOf( i - trennstriche ) );

            checkBoxes.add( checkBox );

            x += 2;
            panel.add( checkBox, cc.xy( x, y ) );

            // Der Steigerungsfaktor
            // *********************
            final JLabel faktorLabel = new JLabel();

            steigerungsfaktoren.add( faktorLabel );

            x += 2;
            panel.add( faktorLabel, cc.xy( x, y ) );

            // Die Punkte die auf Attacke verteilt wurden
            // ******************************************
            final JSpinner attacke = Util.erzeugeSpinner( 0, 0, 99, 1 );

            attacke.addChangeListener( attackeListener );

            attacke.setName( String.valueOf( i - trennstriche ) );

            attacken.add( attacke );

            x += 2;
            panel.add( attacke, cc.xy( x, y ) );

            // Die Punkte auf Parade
            // *********************
            final JLabel parade = new JLabel( "0" );

            x += 2;
            panel.add( parade, cc.xy( x, y ) );

            paraden.add( parade );

            // Bei Fernkampftalenten und Lanzenreiten sind keine Punkte auf
            // Attacke und Parade zu verteilen.
            // Label und Spinner werden in diesem Fall ausgeblendet.
            if ( ( ! talent.istNahkampfTechnik() )
                    ||  ( talent == Talent.LANZENREITEN ) ) {

                attacke.setVisible( false );

                parade.setVisible( false );
            }

            // Der Spinner zum waehlen der Stufe
            // *********************************
            final JSpinner spinner = Util.erzeugeSpinner( 0, -5, 99, 1 );

            spinner.addChangeListener( actionStufe );

            stufen.add( spinner );

            spinner.setName( String.valueOf( i - trennstriche ) );

            x += 2;
            panel.add( spinner, cc.xy( x, y ) );

            // Die Kosten
            // **********
            final JLabel k = new JLabel( "100 AP" );

            kosten.add( k );

            x += 2;
            panel.add( k, cc.xy( x, y ) );

            // Die Checkbox fuer die Spezielle Erfahrung
            // *****************************************
            if ( steigern ) {

                final JCheckBox se = new JCheckBox();

                se.setName( String.valueOf( i - trennstriche ) );

                se.addActionListener( actionSpezielleErfahrung );

                spezielleErfahrungen.add( se );

                x += 2;
                panel.add( se, cc.xy( x, y ) );
            }
        }

        final JScrollPane scrollPane = Util.erzeugeScrollPane( panel );
        
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );

        add( scrollPane, BorderLayout.CENTER );
    }

    /**
     * @return das Layout fuer die Benutzeroberflaeche.
     */
    private FormLayout erzeugeLayout() {

        final String spaltenLayout;
        //                 Leittalent  Talent     Faktor       AT         PA
        spaltenLayout = "7dlu, P, 4dlu, F:P, 4dlu, C:P, 4dlu, C:P, 4dlu, C:P, " +
        //            Stufe     Kosten     SE
                "4dlu, C:P, 4dlu, P, 4dlu, P, 7dlu";

        // Jetzt das Zeilenlayout festlegen.
        // Es gibt eine Zeile fuer die Ueberschrift
        // eine Zeile mit einem Trennstrich unter der Ueberschrift.
        // eine Zeile fuer jedes Talent
        // und alle 5 Zeilen wird ein Trennstrich eingefuegt.
        final int extraZeilen = 1 + ( talente.size() / 4 );

        final StringBuilder zeilenLayout = new StringBuilder();

        zeilenLayout.append( "7dlu, P," );

        for ( int i = 0, n = talente.size() + extraZeilen ; i < n ; ++i ) {

            zeilenLayout.append( " 4dlu, P," );
        }

        zeilenLayout.append( " 7dlu" );

        return new FormLayout( spaltenLayout,
                               zeilenLayout.toString() );

    }

    /**
     * Initialisiert {@link #abhaengigkeiten}.
     */
    private void initAbhaengigkeiten() {

        abhaengigkeiten.clear();
        
        for( int i = talente.size()-1; i >= 0 ; --i ) {

            final Talent talent = talente.get( i );

            for ( final Object regelelement : talent.getAbhaengigkeiten() ) {

                setAbhaengigkeit( regelelement, i );
            }
        }
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();

        initAbhaengigkeiten();
        
        for ( int i = talente.size()-1; i >= 0 ; --i ) {

            final Kampftechnik talent = (Kampftechnik) talente.get( i );
            
            checkBoxes.get( i ).setText( talent.getName() );
            
            if ( held.hatLeittalent( talente.get( i ) ) ) {

                leittalente.get( i ).setIcon( ICON_LEITTALENT );
                leittalente.get( i ).setToolTipText( I18n.i18n( 
                        TEXT_LEITTALENT ) );

            } else {

                leittalente.get( i ).setIcon( null );
                leittalente.get( i ).setToolTipText( null );
            }
            
            if ( ( ! talent.istNahkampfTechnik() )
                    ||  ( talent == Talent.LANZENREITEN ) ) {

                attacken.get( i ).setVisible( false );

                paraden.get( i ).setVisible( false );
            
            } else {
                
                attacken.get( i ).setVisible( true );
                
                paraden.get( i ).setVisible( true );
            }
            
            update( i );
        }
    }
    
    /**
     * Macht ein Talent von einem Regelelement abhaengig.
     *
     * @param regelelement
     *            von dem das Talent abhaengig ist.
     * @param index
     *            des Talents in {@link #talente}.
     */
    private void setAbhaengigkeit(
            final Object regelelement,
            final int index ) {

        if ( ! abhaengigkeiten.containsKey( regelelement ) ) {

            abhaengigkeiten.put( regelelement, new ArrayList< Integer >() );
        }

        abhaengigkeiten.get( regelelement ).add( index );
    }

    /**
     * Bestimmt den Helden dessen Werte im Panel angezeigt und editiert werden.
     *
     * @param derHeld der vom Panle bearbeitet wird.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        held.addObserver( this );

        for ( int i = talente.size()-1; i >= 0 ; --i ) {

            if ( held.hatLeittalent( talente.get( i ) ) ) {

                leittalente.get( i ).setIcon( ICON_LEITTALENT );
                leittalente.get( i ).setToolTipText( I18n.i18n( 
                        TEXT_LEITTALENT ) );

            } else {

                leittalente.get( i ).setIcon( null );
                leittalente.get( i ).setToolTipText( null );

            }

            update( i );
        }
    }

    /**
     * Aktualisiert eine Zeile in der Benutzeroberflaeche.
     *
     * @param index
     *            der zu aktualisierenden Zeile.
     */
    private void update( final int index ) {

        final Talent talent = talente.get( index );

        final boolean hatTalent = held.hat( talent );

        // Aktiviert
        // *********
        final JCheckBox checkBox = checkBoxes.get( index );

        checkBox.setSelected( hatTalent );

        // Regelkonflikte
        // **************
        final KonfliktBeschreibung beschreibung = talent.pruefen( held );

        if ( beschreibung.enthaeltKonflikt() ) {

            checkBox.setForeground( Util.FEHLERFARBE );
            checkBox.setToolTipText( beschreibung.toHtml() );

        } else {

            checkBox.setForeground( getForeground() );
            checkBox.setToolTipText( null );

        }

        // Vorgegeben
        // **********
        if ( held.hatVorgegeben( talent ) ) {

            checkBox.setFont( fetteSchrift );

        } else {

            checkBox.setFont( getFont() );

        }

        // Steigerungsfaktor
        // *****************
        int stufe;

        if ( hatTalent ) {

            stufe = held.getStufe( talent );

        } else {

            stufe = 0;

        }

        final Lernmethode lernmethode;

        if ( steigern && ( null != lernmethoden.getLernmethode() ) ) {

            lernmethode = lernmethoden.getLernmethode();

        } else {

            lernmethode = Lernmethode.KEINE;
        }

        final int faktor = talent.getSteigerungsspalte(
                held,
                stufe,
                lernmethode );

        steigerungsfaktoren.get( index ).setText( SKT.toString( faktor ) );

        // Stufe, AT und PA
        // ****************
        final JSpinner spinner = stufen.get( index );

        final int at;

        final String pa;

        if ( hatTalent ) {

            final TalentWert wert = held.getWert( talent );

            at = wert.getAttacke();

            pa = String.valueOf( wert.getParade( held ) );

            stufe = held.getStufe( talent );

        } else {

            at = 0;

            pa = "0";

            stufe = 0;

        }

        attacken.get( index ).setValue( at );

        paraden.get( index ).setText( pa );

        spinner.setValue( stufe );

        spinner.setEnabled( hatTalent );

        attacken.get( index ).setEnabled( hatTalent );


        final int k;
        if ( hatTalent ) {

            final TalentWert wert = held.getWert( talent );

            k = wert.getSteigerungskosten( held, lernmethode );

        } else {

            k = talent.getAktivierungskosten( held, lernmethode );

        }

        kosten.get( index ).setText( String.format( I18n.i18n( AP ), k ) );

        // Spezielle Erfahrung
        // *******************
        if ( steigern ) {

            final JCheckBox se = spezielleErfahrungen.get( index );

            if ( hatTalent ) {

                final TalentWert wert = held.getWert( talent );

                se.setSelected( wert.hatSpezielleErfahrung() );

                se.setEnabled( true );

            } else {

                se.setSelected( false );

                se.setEnabled( false );
            }
        }

    }
    /**
     * Aktualisert die Anzeige wenn eine Veraenderung am Helden durchgefuehrt
     * wurde.
     *
     * @param o ungenutzt.
     * @param arg das Regelelement des Helden das veraendert wurde.
     */
    public final void update( final Observable o, final Object arg ) {

        if ( Lernmethoden.LERNMETHODE.equals( arg ) ) {

            for ( int i = 0, n = talente.size() ; i < n ; ++i ) {

                update( i );
            }

        } else if ( abhaengigkeiten.containsKey( arg ) ) {

            for ( final int i : abhaengigkeiten.get( arg ) ) {

                update( i );
            }
        }
    }

    /**
     * Lauscht auf Veraenderungen an den Spinnern mit denen die Punkte auf
     * Attacke verteilt werden.
     *
     * @author Bernhard Jung
     */
    private class AttackeListener implements ChangeListener {

        public void stateChanged( final ChangeEvent e ) {

            final JSpinner spinner = (JSpinner) e.getSource();

            final int i = Integer.valueOf( spinner.getName() );

            final Talent talent = talente.get( i );

            final int stufe = (Integer) spinner.getValue();

            if ( held.hat( talent ) ) {

                held.getWert( talent ).setAttacke( stufe );

            }
        }

    }


    /**
     * Wird aufgerufen wenn eine der CheckBoxen angeklickt wird.
     * Aktiviert und deaktiver die Talente eines Helden.
     */
    private class ActionDeAktivierenErschaffung implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JCheckBox checkBox = (JCheckBox) e.getSource();

            final int index = Integer.valueOf( checkBox.getName() );

            final Talent talent = talente.get( index );

            if ( checkBox.isSelected() ) {
                // Der Benutzer will das Talent aktivieren.

                if ( held.hat( talent ) ) {
                    // Sonderfall Spezial-Talent mit negativer vorgegebener Stufe:
                    // Der Benutzer hat gerade auf die Checkbox
                    // geklickt um ein Talent zu aktivieren. Wie die Abfrage von
                    // held.hat( talent ) aber just ergeben hat besitzt der Held
                    // das Talent schon. Es muss sich dabei also um ein
                    // Spezial-Talent mit negativer Stufe handeln.
                    // Das Talent ist nur noch als aktivert zu markieren und auf
                    // Stufe 0 zu setzen.
                    final TalentWert wert = held.getWert( talent );

                    wert.setAktiviert( true );

                    wert.setStufe( 0 );

                    return;

                }

                // Der normale Fall. Der Held erhaelt das Talent auf Stufe 0 oder
                // der vorgegebenen Mindeststufe neu.
                final int stufe;
                if ( held.hatVorgegeben( talent ) ) {

                    stufe = held.getMindeststufe( talent );

                } else {

                    stufe = 0;

                }

                held.einfuegen( new TalentWert( talent, stufe ) );

            } else {
                // Der Benutzer will das Talent deaktivieren.

                if ( held.hatVorgegeben( talent )
                        && held.getMindeststufe( talent ) < 0
                        && ( ! talent.istBasisTalent() ) ) {
                    // Sonderfall Spezial-Talent mit negativer vorgegebener Stufe:
                    // Das Talent wird nicht entfernt, sonder nur als deaktivert
                    // markiert. Die Stufe wird auf den vorgegebenen negativen Wert
                    // gesetzt oder bleibt auf 0 wenn sie gerade 0 ist.
                    final TalentWert wert = held.getWert( talent );

                    wert.setAktiviert( false );

                    int stufe = wert.getStufe( held );

                    if ( 0 < stufe ) {
                        // Aktuell ist die Stufe positiv, da das Talent deaktiviert
                        // werden soll wird es auf die negative vorgegebene Stufe
                        // gestellt.
                        stufe = held.getMindeststufe( talent );

                    }

                    wert.setStufe( stufe );

                } else {
                    // Der normale Fall. Das Talent wird einfach entfernt.
                    held.entfernen( talent );

                }

            }
        }

    }


    /**
     * Wird aufgerufen wenn eine der CheckBoxen angeklickt wird.
     * Aktiviert und deaktiver die Talente eines Helden.
     */
    private class ActionDeAktivierenSteigern implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JCheckBox checkBox = (JCheckBox) e.getSource();

            final int index = Integer.valueOf( checkBox.getName() );

            final Talent talent = talente.get( index );

            if ( checkBox.isSelected() ) {

                held.aktivieren( talent, lernmethoden.getLernmethode() );

            } else {

                held.deaktivieren( talent );
            }
        }
    }

    /**
     * Wird aufgerufen wenn der Wert in einem der Spinner veraendert wird.
     */
    private class ActionStufeErschaffung implements ChangeListener {

        public void stateChanged( final ChangeEvent e ) {

            final JSpinner spinner = (JSpinner) e.getSource();

            final int i = Integer.valueOf( spinner.getName() );

            final Talent talent = talente.get( i );

            if ( held.hat( talent ) ) {

                final int stufe = (Integer) spinner.getValue();

                final TalentWert wert = held.getWert( talent );

                wert.setStufe( stufe );
            }
        }
    }

    /**
     * Wird aufgerufen wenn der Wert in einem der Spinner veraendert wird.
     */
    private class ActionStufeSteigern implements ChangeListener {

        public void stateChanged( final ChangeEvent e ) {

            final JSpinner spinner = (JSpinner) e.getSource();

            final int i = Integer.valueOf( spinner.getName() );

            final Talent talent = talente.get( i );

            if ( held.hat( talent ) ) {

                final int stufe = (Integer) spinner.getValue();

                held.setStufe( talent, stufe, lernmethoden.getLernmethode() );
            }
        }
    }
}
