/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;

import javax.swing.JTabbedPane;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;

public class PanelRasKulProf
extends ResetbaresPanel 
implements Uebersetzbar {

    /**
     * Der Titel der Registerkarte mit den Professionen.
     */
    private static final String PROFESSION = "PanelRasKulProf.Profession";

    /**
     * Der Titel der Registerkarte mit den Kulturen. 
     */
    private static final String KULTUR = "PanelRasKulProf.Kultur";

    /**
     * Der Titel der Registerkarte mit den Rassen.
     */
    private static final String RASSE = "PanelRasKulProf.Rasse";

    /** Das Panel mit dem die Kultur gewaehlt wird. */
    private final PanelKultur panelKultur;

    /** Das Panel mit dem die Profession gewaehlt wird. */
    private final PanelProfession panelProfession;

    /** Das Panel mit dem die Rasse gewaehlt wird. */
    private final PanelRasse panelRasse;
    
    /**
     * Uebersetzt die Texte der Benutzeroberflaeche.
     */
    private final Uebersetzer uebersetzer;
    
    PanelRasKulProf() {

        super( new BorderLayout() );

        uebersetzer = new Uebersetzer();
        
        panelProfession = new PanelProfession();
        uebersetzer.einfuegen( panelProfession );
        resetter.einfuegen( panelProfession );
        
        panelKultur = new PanelKultur( panelProfession );
        uebersetzer.einfuegen( panelKultur );
        resetter.einfuegen( panelKultur );

        panelRasse = new PanelRasse( panelKultur );
        uebersetzer.einfuegen( panelRasse );
        resetter.einfuegen( panelRasse );

        final JTabbedPane tabbedPane = new JTabbedPane();
        resetter.einfuegen( tabbedPane );

        uebersetzer.erzeugeTab( tabbedPane, RASSE, panelRasse );
        
        uebersetzer.erzeugeTab( tabbedPane, KULTUR, panelKultur );
        
        uebersetzer.erzeugeTab( tabbedPane, PROFESSION, panelProfession );

        add( tabbedPane );
    }

    /**
     * Meldet das Panel als Observer des Helden ab.
     */
    public final void abmeldenVonHeld() {

        panelRasse.abmeldenVonHeld();

        panelKultur.abmeldenVonHeld();

        panelProfession.abmeldenVonHeld();
    }
    
    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Aendert den {@link Held} fuer den Rasse, Kultur und Profession gewaehlt
     * werden. Meldet das Panel als Observer beim Helden an.
     *
     * @see #abmeldenVonHeld()
     * @param held
     *            der gerade erschaffen wird.
     */
    public final void setHeld( final Held held ) {

        panelRasse.setHeld( held );

        panelKultur.setHeld( held );

        panelProfession.setHeld( held );
    }
}
