/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.action;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import javax.swing.JFileChooser;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.IXMLParser;
import net.n3.nanoxml.IXMLReader;
import net.n3.nanoxml.StdXMLReader;
import net.n3.nanoxml.XMLParserFactory;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.heldio.FactoryHeldXmlIn;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.steigern.PanelSteigern;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.LookAndFeelAktualisierbar;
import de.bernhardjung.dsaprogramm.util.Util;

public class ActionLadenHeldXml 
extends CharGenAction 
implements LookAndFeelAktualisierbar {

    /** Der Name des Icons. */
    private static final String ICON = "laden";

    /** Der Parser zum laden der Dateien. */
    private IXMLParser parser;

    /** Erzeugt den Helden aus dem XML-Dokument. */
    private final FactoryHeldXmlIn factory;

    /** Zum Auswaehlen der zu ladenden Datei. */
    private JFileChooser fileChooser;

    public ActionLadenHeldXml( final Hauptpanel dasHauptpanel ) {

        super( dasHauptpanel,
                "ActionLadenHeldXml.Name", 
                ICON,
                "ActionLadenHeldXml.Tasten",
                "ActionLadenHeldXml.Mnemonic" );

        try {
            
            parser = XMLParserFactory.createDefaultXMLParser();
        
        } catch ( final Exception e ) {
            e.printStackTrace();
        }

        factory = new FactoryHeldXmlIn();
    }

    protected void actionPerformed() {
 
        if ( ! hauptpanel.kannPanelWechseln() ) {
            
            return;
        }
        
        if ( null == fileChooser ) {
            
            fileChooser = Util.erzeugeFileChooser( ".dcg" );
        }
        
        final File verzeichnis =
            new File( Einstellungen.getInstanz().getHeldenXmlVerzeichnis() );
        
        fileChooser.setCurrentDirectory( verzeichnis );
        
        final int laden = fileChooser.showDialog( 
                hauptpanel, 
                I18n.i18n( "ActionLadenHeldXml.Name" ) );

        if ( laden == JFileChooser.APPROVE_OPTION ) {

            final File datei = fileChooser.getSelectedFile();

            if ( datei.exists() ) {

                try {

                    final InputStream in = new FileInputStream( datei );

                    final IXMLReader reader = new StdXMLReader( in );

                    parser.setReader( reader );

                    final IXMLElement wurzelElement = (IXMLElement) parser.parse();

                    final Held held = factory.erzeugen( wurzelElement );

                    final PanelSteigern panelSteigern =
                        hauptpanel.getPanelSteigern();

                    panelSteigern.setHeld( held, false );

                    hauptpanel.anzeigen( panelSteigern );
                    
                } catch (final Exception ex) {

                    DialogFehlermeldung.zeigeDialog( 
                            I18n.i18n( "ActionLadenHeldXml.Fehler" ), 
                            ex );

                }
            }
        }
    }

    @Override
    public void neuUebersetzen() {
        
        super.neuUebersetzen();
    }
    
    public void aktualisiereLookAndFeel() {
        
        fileChooser = null;
    }
}
