/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.util.Beobachtbar;

class BeobachtbareListe< T > extends Beobachtbar {

    /**
     * Wird an die Beobachter gesendet, wenn die Liste geleert wird.
     */
    public static final Object CLEAR = new Object();

    private final List< T > elemente;

    BeobachtbareListe() {

        elemente = new ArrayList< T >();
    }

    /**
     * Fuegt der Liste ein Element hinzu.
     *
     * @param element
     *            das neue Element fuer die Liste. Das Element wird hinten an
     *            die Liste angehaengt.
     * @return {@code true}
     */
    public final boolean add( final T element ) {

        elemente.add( element );

        meldeBeobachtern( element );

        return true;
    }

    /**
     * Entfernt alle Elemente aus der Liste.
     */
    public final void clear() {

        elemente.clear();

        meldeBeobachtern( CLEAR );
    }

    /**
     * Prueft ob ein Element in der Liste vorhanden ist.
     *
     * @param o
     *            Das Element, von dem geprueft werden soll, ob es in der Liste
     *            ist.
     * @return {@code true}, wenn das Element in der Liste ist, {@code false}
     *         sonst.
     */
    public final boolean contains( final T o ) {
        return elemente.contains( o );
    }

    /**
     * Liefert ein Element an einen bestimmten Position in der Liste.
     *
     * @param index
     *            des gesuchten Elements.
     * @return das gesuchte Element.
     */
    public final T get( final int index ) {
        return elemente.get( index );
    }

    /**
     * Liefert ein Objekt aus der Liste.
     * @param element
     *            in der Liste, das gesucht ist.
     * @return das gesucht Objekt oder {@code null}, wenn das Objekt nicht in
     *         der Liste ist.
     */
    public final T get( final T element ) {

        final int index = elemente.indexOf( element );

        if ( index >= 0 ) {

            return elemente.get( index );

        } else {

            return null;
        }

    }

    /**
     * @return alle in der Liste gespeicherten Elemente.
     */
    public final Collection< T > getElemente() {
        return new ArrayList< T >( elemente );
    }

    /**
     * Entfernt ein Element aus der Liste.
     *
     * @param o
     *            das Element, das aus der Liste entfernt werden soll.
     * @return {@code true}, wenn das Element in der Liste war und entfernt
     *         wurde, {@code false} falls das Element nicht in der Liste ist.
     */
    public final boolean remove( final Object o ) {

        final boolean entfernt = elemente.remove( o );

        if ( entfernt ) {

            meldeBeobachtern( o );
        }

        return entfernt;
    }

    /**
     * @return die Anzahl an Elementen in der Liste.
     */
    public final int size() {
        return elemente.size();
    }

}
