/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFest;
import de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten.TalentListe;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.ComparatorInteger;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelMetaTalentbonusFestKultur
extends ResetbaresPanel 
implements Uebersetzbar {

    /**
     * Die Ueberschrift fuer den Abschnitt in dem die Stufenboni gewaehlt
     * werden.
     */
    private static final String BONI =
        "PanelMetaTalentbonusFest.Boni";

    /** Der Text auf dem Knopf mit dem ein Stufenbonus hinzugefuegt wird. */
    private static final String EINFUEGEN =
        "PanelMetaTalentbonusFest.Einfuegen";

    /** Die Ueberschrift fuer die Talente. */
    private static final String TALENTE =
        "PanelMetaTalentbonusFest.Talente";

    /**
     * Der Text auf dem Knopf, mit dem man den im Panel gewaehlten Bonus, in die
     * Liste der Talentboni uebernimmt.
     */
    private static final String UEBERNEHMEN =
        "PanelMetaTalentbonusFest.Uebernehmen";

    /** Ein Spinner, in dem ein Stufenbonus eingestellt werden kann., */
    private final JSpinner bonus;

    /** Die vom Benutzer, fuer den Bonus, ausgewaehlten Stufenboni. */
    private final JList gewaehlteBoniListe;

    /** Das ListModel fuer die Liste der Stufenboni. */
    private final CharGenListModel< Integer, Integer > gewaehlteBoniModel;

    
    
    /** 
     * Die Liste von Talentboni, in die man, ueber dieses Panel, neue 
     * Talentboni einfuegt.
     */
    private final CharGenListModel< Talentbonus, Talentbonus > talentboni;

    /**
     * Der Knopf mit dem man einen im Panel eingestellten Talentbonus in die
     * Liste der Talentboni uebernimmt.
     */
    private final JButton uebernehmen;
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;

    PanelMetaTalentbonusFestKultur(
            final CharGenListModel< Talentbonus, Talentbonus > dieTalentboni ) {

        uebersetzer = new Uebersetzer();
    
        
        
        bonus = Util.erzeugeSpinner( 0, -9, 99, 1 );

        gewaehlteBoniModel = new CharGenListModel< Integer, Integer >(
               new ComparatorInteger() );

        gewaehlteBoniListe = uebersetzer.erzeugeListe( gewaehlteBoniModel );
        gewaehlteBoniListe.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        gewaehlteBoniListe.addListSelectionListener(
                new ListSelectionListener() {
                    public void valueChanged( final ListSelectionEvent e ) {
                        if ( e.getValueIsAdjusting() ) {
                            return;
                        }
                        final int index = gewaehlteBoniListe.getSelectedIndex();
                        if ( index >= 0 ) {
                            gewaehlteBoniModel.removeElementAt( index );
                            pruefen();
                        }
                    }
                });

        talentboni = dieTalentboni;
        
        uebernehmen = uebersetzer.erzeugeButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {

                final TalentbonusFest bonus = new TalentbonusFest(
                        TalentListe.FREMDSPRACHEN,
                        gewaehlteBoniModel.getElements() );

                talentboni.addElement( bonus );

                gewaehlteBoniModel.clear();

                pruefen();
            }
        });
        uebernehmen.setEnabled( false );

        aufbauenBenutzeroberflaeche();
    }

    private void aufbauenBenutzeroberflaeche() {

        final JButton einfuegen = uebersetzer.erzeugeButton( EINFUEGEN );
        einfuegen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                gewaehlteBoniModel.addElement( bonus.getValue() );
                pruefen();
            }
        });

        final JPanel spinnerPanel = new JPanel(
                new FormLayout( "P, 4dlu, P", "P" ) );

        final CellConstraints cc = new CellConstraints();

        spinnerPanel.add( bonus, cc.xy( 1, 1 ) );

        spinnerPanel.add( einfuegen, cc.xy( 3, 1) );

        final String spalten = "7dlu, F:P:G, 7dlu";

        final String zeilen =
            "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, F:P:G, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        final JPanel inhalt = new JPanel( layout );

        inhalt.add( uebersetzer.erzeugeSeparator( TALENTE ), cc.xy( 2, 2 ) );

        inhalt.add( new JLabel( TalentListe.FREMDSPRACHEN.getBeschreibung() ),
                    cc.xy( 2, 4 ) );

        inhalt.add( uebersetzer.erzeugeSeparator( BONI ), cc.xy( 2, 6 ) );

        inhalt.add( spinnerPanel, cc.xy( 2, 8 ) );

        inhalt.add( Util.erzeugeScrollPane( gewaehlteBoniListe, 150, 60 ),
                    cc.xy( 2, 10 ) );

        inhalt.add( uebernehmen, cc.xy( 2, 12 ) );
        
        final JScrollPane scrollPane = Util.erzeugeScrollPane( inhalt );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }
    
    /**
     * Uebersetzt die Benutzeroberflaeche neu.
     */
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    private void pruefen() {

        final int anzahlBoni = gewaehlteBoniModel.getSize();

        uebernehmen.setEnabled( anzahlBoni > 0 );
    }

    /** Loescht die gewaehlten Boni. */
    public void eingabenLoeschen() {
        gewaehlteBoniModel.clear();
    }
}
