/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ListSelectionModel;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Zauberbonus;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.zauber.Zaubergruppe;

class PanelMetaZauberboni extends ResetbaresPanel implements Uebersetzbar {

    /** Die Aufschrift des Knopfes, zum loeschen eines Bonus. */
    private static final String ENTFERNEN =
        "PanelMetaZauberboni.Entfernen";
    
    

    /** Uebersetzt die Texte auf dem Panel. */
    private final Uebersetzer uebersetzer;
    
    /**
     * Eine Liste aller bisher gewaehlten Zauberboni.
     */
    private final JList zauberboniListe;

    /** Das ListModel fuer die Liste {@link #zauberboniListe}. */
    private final CharGenListModel< Zauberbonus, Zauberbonus > zauberboniModel;

    PanelMetaZauberboni() {

        uebersetzer = new Uebersetzer();
        
        
        
        zauberboniModel = new CharGenListModel< Zauberbonus, Zauberbonus >();
        
        zauberboniListe = uebersetzer.erzeugeListe( zauberboniModel );
        zauberboniListe.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

        aufbauenBenutzeroberflaeche();
    }

    public final void anzeigen( final Heldenbaustein baustein ) {

        zauberboniModel.setElements( baustein.getZauberboni() );
    }

    private void aufbauenBenutzeroberflaeche() {

        final JButton entfernen = uebersetzer.erzeugeButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                entfernen( e );
            }
        });

        final JTabbedPane tabbedPane = new JTabbedPane();
        resetter.einfuegen( tabbedPane );
        
        for ( final Zaubergruppe gruppe : Zaubergruppe.getZaubergruppen() ) {
            
            final PanelMetaZauberbonus panel = new PanelMetaZauberbonus( 
                    zauberboniModel,
                    gruppe );
            resetter.einfuegen( panel );
            
            uebersetzer.erzeugeTab( 
                    tabbedPane,
                    gruppe.getLokalisierungsschluesselKuerzel(),
                    panel );
        }

        setLayout( new FormLayout( "7dlu, F:150px:G, 4dlu, F:P:G, 7dlu",
                                   "7dlu, F:200px:G, 4dlu, P, 7dlu" ) );

        final CellConstraints cc = new CellConstraints();

        add( new JScrollPane( zauberboniListe ), cc.xy( 2, 2 ) );

        add( entfernen, cc.xy( 2, 4 ) );

        add( tabbedPane, cc.xywh( 4, 2, 1, 3 ) );
    }

    /**
     * Wird aufgerufen, wenn der gewaehlte Bonus aus der Liste
     * {@link #zauberboniModel} entfernt werden soll.
     */
    private void entfernen( final ActionEvent e ) {

        final int index = zauberboniListe.getSelectedIndex();

        if ( index >= 0 ) {

            zauberboniModel.removeElementAt( index );
        }
    }

    /**
     * @return eine Liste der Talentboni.
     */
    public List< Zauberbonus > getBoni() {
        return zauberboniModel.getElements();
    }
    
    
    /** Uebersetzt die Texte auf dem Panel neu. */
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    /** Loescht alle Boni aus dem Panel. */
    public void eingabenLoeschen() {
        zauberboniModel.clear();
    }
}
