/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.bonus.Zauberbonus;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.zauber.Zauber;

class PanelMetaZauberbonus extends ResetbaresPanel implements Uebersetzbar {

    /**
     * Der Text neben dem Spinner mit dem der Bonus eingestellt wird.
     */
    private static final String BONUS  =
        "PanelMetaZauberbonus.Bonus";

    /**
     * Der Text der Checkbox mit der angegeben werden kann, dass der Held den
     * Zauber als Hauszauber erhalten soll.
     */
    private static final String HAUSZAUBER =
        "PanelMetaZauberbonus.Hauszauber";

    /**
     * Der Text auf dem Knopf, mit dem man den im Panel gewaehlten Bonus, in die
     * Liste der Talentboni uebernimmt.
     */
    private static final String UEBERNEHMEN =
        "PanelMetaZauberbonus.Uebernehmen";

    /**
     * Der Text auf dem Knopf, mit dem man eine Variante eingeben kann.
     */
    private static final String VARIANTE =
        "PanelMetaZauberbonus.Variante";

    /** Ein Spinner, in dem ein Stufenbonus eingestellt werden kann., */
    private final JSpinner bonus;

    /** Die Liste der Zauber. */
    private final JList zauberListe;

    /**
     * Mit dieser Checkbox kann angegeben werden, ob es sich bei dem erhaltenen
     * Zauber um einen Hauszauber handelt.
     */
    private final JCheckBox hauszauber;

    /**
     * Die Liste von Zauberboni, in die man, ueber dieses Panel, neue Boni
     * einfuegt.
     */
    private final CharGenListModel< Zauberbonus, Zauberbonus > zauberboni;

    
    
    /**
     * Der Knopf mit dem man einen im Panel eingestellten Zauberbonus in die
     * Liste der Zauberboni uebernimmt.
     */
    private final JButton uebernehmen;

    /** Uebersetzt die Texte auf dem Panel*/
    private final Uebersetzer uebersetzer;
    
    /**
     * Ein Textfeld in das einen Variante eingegeben werden kann.
     */
    private final JTextField variante;

    /** Der Text neben dem Textfeld fuer die Variante. */
    private final JLabel varianteLabel;

    PanelMetaZauberbonus(
            final CharGenListModel< Zauberbonus, Zauberbonus > dieZauberboni,
            final List< Zauber > dieZauber ) {

        uebersetzer = new Uebersetzer();
    
        
        
        bonus = Util.erzeugeSpinner( 0, 0, 99, 1 );

        hauszauber = uebersetzer.erzeugeCheckBox( HAUSZAUBER );

        uebernehmen = uebersetzer.erzeugeButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });
        uebernehmen.setEnabled( false );

        variante = new JTextField();
        variante.setEnabled( false );

        varianteLabel = uebersetzer.erzeugeLabel( VARIANTE );
        varianteLabel.setEnabled( false );

        zauberboni = dieZauberboni;

        zauberListe = uebersetzer.erzeugeListe( 
                dieZauber, 
                Zauber.NAME_COMPARATOR );
        zauberListe.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        zauberListe.addListSelectionListener(
                new ListSelectionListener(){
                    public void valueChanged( final ListSelectionEvent e ) {
                        if ( e.getValueIsAdjusting() ) {
                            return;
                        }
                        final int index = zauberListe.getSelectedIndex();

                        if ( 0 <= index ) {

                            uebernehmen.setEnabled( true );

                            final Zauber zauber =
                                (Zauber) zauberListe.getSelectedValue();

                            variante.setEnabled( zauber.hatVarianten() );
                            varianteLabel.setEnabled( zauber.hatVarianten() );

                        } else {

                            uebernehmen.setEnabled( false );

                            variante.setEnabled( false );
                            varianteLabel.setEnabled( false );
                        }
                    }
                });

        aufbauenBenutzeroberflaeche();
    }

    private void aufbauenBenutzeroberflaeche() {

        final JScrollPane zauberListeScrollPane = 
            new JScrollPane( zauberListe );
        resetter.einfuegen( zauberListeScrollPane );
        
        final String spalten = "7dlu, F:P:G, 7dlu";

        final String zeilen = "7dlu, F:150px:G, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        final JPanel inhalt = new JPanel( layout );

        final CellConstraints cc = new CellConstraints();

        inhalt.add( zauberListeScrollPane, cc.xy( 2, 2 ) );

        final FormLayout layout2 = new FormLayout( "P, 4dlu, P:G" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout2 );

        builder.append( varianteLabel, variante );

        inhalt.add( builder.getPanel(), cc.xy( 2, 4 ) );

        final FormLayout layout3 = new FormLayout( "P, 4dlu, P, P:G" );

        final DefaultFormBuilder builder2 = new DefaultFormBuilder( layout3 );

        builder2.append( uebersetzer.erzeugeLabel( BONUS ), bonus );

        inhalt.add( builder2.getPanel(), cc.xy( 2, 6 ) );

        inhalt.add( hauszauber, cc.xy( 2, 8 ) );

        inhalt.add( uebernehmen, cc.xy( 2, 10 ) );
        
        final JScrollPane scrollPane = 
            Util.erzeugeScrollPane( inhalt );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    /** Uebersetzt die Texte auf dem Panel neu. */
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Uebernimmt den im Panel gewaehlten Bonus in die Liste der Boni.
     */
    private void uebernehmen() {

        final int index = zauberListe.getSelectedIndex();

        if ( 0 <= index ) {

            final Zauber zauber = (Zauber) zauberListe.getSelectedValue();

            final String variante;

            if ( zauber.hatVarianten() ) {

                variante = this.variante.getText().trim();

                zauberboni.addElement( new Zauberbonus( 
                        zauber,
                        variante,
                        (Integer) bonus.getValue(),
                        hauszauber.isSelected() ) );
            } else {

                zauberboni.addElement( new Zauberbonus( 
                        zauber,
                        (Integer) bonus.getValue(),
                        hauszauber.isSelected() ) );
            }
        }
    }
}
