/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente;

public class Schrift extends Talent {
    
    /**
     * Die Komplexitaet ist die Hoechststufe auf die die Schrift
     * gesteigert werden kann.
     */
    private int komplexitaet;
   
   /**
     * @param dieId der Sprache.
     */
    Schrift( final String dieId ) {
        
        super( dieId, Talentgruppe.SCHRIFTEN, Typ.SPEZIAL );
       
        komplexitaet = -1;
    }
    
    /**
     * Liefert die Komplexitaet einer Schrift. Die Komplexitaet gibt die 
     * Hoechststufe an bis zu der die Schrift gesteigert werden darf.
     * 
     * @return die Komplexitaet der Schrift.
     */
    public int getKomplexitaet() {
        return komplexitaet;
    }
    
    /**
     * Bestimmt die Komplexitaet der Schrift.
     * Die Methode sollte nur einmal aufgerufen werden um die Schrift zu 
     * initialisieren.
     * 
     * @param dieKomplexitaet der Schrift.
     * @throws AssertionError wenn die Komplexitaet schon einmal gesetzt wurde.
     */
    void setKomplexitaet( final int dieKomplexitaet ) {
        
        assert( -1 == komplexitaet ) : String.format(
                "Die Komplexitaet von %1$s sollte ein zweites mal gesetzt " +
                "werden.\n" +
                "bestehnden Komplexitaet: %2$s\n" +
                "neue Komplexitaet: %3$s",
                getName(),
                komplexitaet,
                dieKomplexitaet );
            
        komplexitaet = dieKomplexitaet;
    }
}
