/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;

class VoraussetzungKampftechnik extends Voraussetzung {

    /** 
     * Die maximale Anzahl an Punkten die Attacke und Parade auseinander 
     * liegen duerfen.
     */
    private static final int MAX_DIFFERENZ = 5;

    /** @return eine leere Liste. */
    @SuppressWarnings("unchecked")
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }
    
    public KonfliktBeschreibung pruefen( 
            final Held held, 
            final Talent talent ) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        if ( held.hat( talent ) ) {

            final int stufe = held.getStufe( talent );
            
            final TalentWert wert = held.getWert( talent );
            
            final int attacke = wert.getAttacke();
            
            final int parade = wert.getParade( held );
            
            final int differenz = Math.abs( attacke - parade );
            
            if ( attacke > stufe ) {
                
                beschreibung.einfuegen( 
                        I18n.i18n( "VoraussetzungKampftechnik.ueberStufe" ) );
                
            } else if ( MAX_DIFFERENZ < differenz ) {
                
                beschreibung.einfuegen( 
                        I18n.i18n( "VoraussetzungKampftechnik.Differenz" ) );
                
            } 
        }
        
        return beschreibung;
    }

}
