/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JComboBox;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;

public class Resetter {

    private final List< JComboBox > comboBoxen;
    
    private final List< JScrollPane > scrollPanes;
    
    private final Map< JSpinner, Object > spinner;
    
    private final List< Resetbar > resetbare;
    
    private final List< JTabbedPane > tabbedPanes;
    
    public Resetter() {
    
        comboBoxen = new ArrayList< JComboBox >();
        
        scrollPanes = new ArrayList< JScrollPane >();
    
        spinner = new HashMap< JSpinner, Object >();
        
        resetbare = new ArrayList< Resetbar >();
        
        tabbedPanes = new ArrayList< JTabbedPane >();
    }
    
    public final void clear() {
        
        comboBoxen.clear();
        
        scrollPanes.clear();
        
        spinner.clear();
        
        resetbare.clear();
        
        tabbedPanes.clear();
    }
    
    public final void einfuegen( final JComboBox comboBox ) {
        
        comboBoxen.add( comboBox );
    }
    
    public final void einfuegen( final Resetbar resetbar ) {
        
        resetbare.add( resetbar );
    }
    
    public final void einfuegen( final JScrollPane scrollPane ) {
    
        scrollPanes.add( scrollPane );
    }
    
    public final void einfuegen( final JSpinner spinner, final Object wert ) {
        
        this.spinner.put( spinner, wert );
    }
    
    public final void einfuegen( final JTabbedPane tabbedPane ) {
        
        tabbedPanes.add( tabbedPane );
    }
    
    public final void reset() {
        
        for ( final JComboBox comboBox : comboBoxen ) {
            
            if ( 0 < comboBox.getItemCount() ) {
                
                comboBox.setSelectedIndex( 0 );
            }
        }
        
        for ( final Resetbar resetbar : resetbare ) {
            
            resetbar.reset();
        }
        
        for ( final JScrollPane scrollPane : scrollPanes ) {
            
            reset( scrollPane.getHorizontalScrollBar() );
            reset( scrollPane.getVerticalScrollBar() );
        }
     
        for ( final JSpinner s : spinner.keySet() ) {
            
            s.setValue( spinner.get( s ) );
        }
        
        for ( final JTabbedPane tabbedPane : tabbedPanes ) {
        
            if ( 0 < tabbedPane.getTabCount() ) {
                
                tabbedPane.setSelectedIndex( 0 );
            }
        }
    }
    
    private void reset( final JScrollBar scrollBar ) {
        
        scrollBar.setValue( 0 );
    }
}
