/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import java.util.Collection;
import java.util.Collections;

class KostenStufe implements Kosten {

    /** Gib an wieviel GP pro {@link #pro} Punkte zu zahlen sind. */
    protected final int kosten;

    /**
     * Beispiel: wenn pro = 2 sind fuer alle 2 Punkte die {@link #kosten} zu
     * bezahlen.
     */
    protected final int pro;

    KostenStufe( final int dieKosten, final int proStufe ) {

        kosten = dieKosten;

        pro = proStufe;
    }

    /** {@inheritDoc} */
    public int getAbbaukosten(
            final Held held,
            final VorNachteilWert wert,
            final Lernmethode lernmethode ) {

        if ( lernmethode == Lernmethode.FREI )
            return 0;

        final int stufe = wert.getStufe();

        return (int) Math.round( -100.0 * stufe * kosten / pro );
    }

    /** @return eine leere Liste. */
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** @return 0 */
    public int getGpKosten() {
        return 0;
    }

    /** @return 0 */
    public int getGpKosten(final VorNachteilWert wert) {

        final int stufe = wert.getStufe();

        return ( kosten * ( stufe / pro ) );
    }

    /**
     * @return die Kosten abhaengig von der Stufe des Wertes.
     */
    public int getGpKosten(
            final Held held,
            final VorNachteilWert wert ) {

        int stufe = wert.getStufe();

        if ( held.hatVorgegeben( wert ) ) {

            stufe -= held.getVorgegeben( wert ).getStufe();

        }

        return ( kosten * ( stufe / pro ) );
    }

    /** {@inheritDoc} */
    public int getSenkungskosten( final Lernmethode lernmethode, final VorNachteilWert wert ) {

        return (int) Math.round( -100.0 * kosten / pro );
    }

    /** @return {@code false} */
    public final boolean istSenkbar( final VorNachteilWert wert ) {
        return true;
    }
}
