/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import java.util.Collection;
import java.util.Collections;

class KostenUnfaehigkeitTalent implements Kosten {

    /**
     * Die GP-Kosten fuer die Talent mit Ausnahme der Kampftechniken und der
     * koerperlichen Talente.
     */
    static final int GP_KOSTEN_ANDERE = -1;

    /** Die GP-Kosten fuer Kampftechniken und koerperliche Talente. */
    static final int GP_KOSTEN_KAMPF_KOERPER = -2;

    public int getAbbaukosten(
            final Held held,
            final VorNachteilWert wert,
            final Lernmethode lernmethode ) {

        if ( lernmethode == Lernmethode.FREI )
            return 0;

        return -100 * getGpKosten( wert );
    }

    /** {@inheritDoc} */
    public int getGpKosten() {
        return GP_KOSTEN_ANDERE;
    }

    /** @return eine leere Liste. */
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** {@inheritDoc} */
    public int getGpKosten( final VorNachteilWert wert ) {

        final Talent talent = (Talent) wert.getVariante().getElement();

        final Talentgruppe talentgruppe = talent.getTalentgruppe();

        if ( talentgruppe.equals( Talentgruppe.KAMPF ) ) {

            return GP_KOSTEN_KAMPF_KOERPER;

        } else if ( talentgruppe.equals( Talentgruppe.KOERPER )
                && !( talent.equals( Talent.SCHWIMMEN )
                        || talent.equals( Talent.SICH_VERSTECKEN )
                        || talent.equals( Talent.SINGEN )
                        || talent.equals( Talent.TANZEN )
                        || talent.equals( Talent.ZECHEN ) ) ) {

            return GP_KOSTEN_KAMPF_KOERPER;

        } else {

            return GP_KOSTEN_ANDERE;

        }
    }

    public int getGpKosten( final Held held, final VorNachteilWert wert ) {

        if ( held.hatVorgegeben( wert ) ) {

            return 0;
        }

        return getGpKosten( wert );
    }

    /** @return 0 */
    public int getSenkungskosten(final Lernmethode lernmethode, final VorNachteilWert wert ) {
        return 0;
    }

    /** @return {@code false} */
    public final boolean istSenkbar( final VorNachteilWert wert ) {
        return false;
    }
}
