package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import java.util.Collection;
import java.util.Collections;

/**
 * Wird fuer Benutzerdefinierte Vor-/Nachteile verwendet, deren Kosten
 * in der Variante gespeichert sind.
 * Entsprechend liest diese Klasse die Kosten aus der Variante aus.
 */
public class KostenVariante implements Kosten {

    /**
     * {@inheritDoc}
     */
    public int getAbbaukosten( Held held, VorNachteilWert wert, Lernmethode lernmethode ) {
        if ( lernmethode == Lernmethode.FREI )
            return 0;
        return getGpKosten( wert ) * 100;
    }

    /**
     * {@inheritDoc}
     */
    public Collection< Object > getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /**
     * {@inheritDoc}
     */
    public int getGpKosten() {
        return 0;
    }

    /**
     * {@inheritDoc}
     */
    public int getGpKosten( VorNachteilWert wert ) {

        Variante variante = wert.getVariante();
        if ( variante.hatStufen() ) {

            int stufe = wert.getStufe();
            int gp = variante.getGpKosten();
            int proStufe = variante.getProStufe();
            return (int) Math.round( stufe * ((double)gp) / proStufe );

        } else {
            return variante.getGpKosten();
        }
    }

    /**
     * {@inheritDoc}
     */
    public int getGpKosten( Held held, VorNachteilWert wert ) {
        return getGpKosten( wert );
    }

    /**
     * {@inheritDoc}
     */
    public int getSenkungskosten( Lernmethode lernmethode, VorNachteilWert wert ) {
        if ( lernmethode == Lernmethode.FREI )
            return 0;
        Variante variante = wert.getVariante();
        if ( variante.hatStufen() ) {
            int gp = variante.getGpKosten();
            int proStufe = variante.getProStufe();
            return ( int ) Math.round( -100.0 * gp / proStufe );
        } else {
            return variante.getGpKosten() * -100;
        }
    }

    /**
     * {@inheritDoc}
     */
    public boolean istSenkbar( VorNachteilWert wert ) {
        return wert.getVariante().hatStufen();
    }
}
